#if !defined(LVSpeechPort_h)
#define LVSpeechPort_h

#include "LV_SRE.h"
#include <iostream>
//*********************************************************************

// This class has is a user-side lightweight wrapper around the "C" API,
// you may bypass this class and call the "C" functions directly.
class LVSpeechPort
{
public:

	// Instantiating an object is not sufficient, you must invoke
	// CreateClient() on the object before it can be used.
	LVSpeechPort();
	virtual ~LVSpeechPort();

	int AddEvent(const char* eventName);
	int AddEventEx(const char* eventName, const char* eventData);

	int AddFieldToRequest(const char* KeyName, const char* String);
	int ClearAllFieldsToRequest();

	int GetDecodeMS(int voicechannel);
	int GetTotalDecodeMS(int voicechannel);

	// Initialize new port, must be done before usable.
	// Vebosity level 
	//		0 will only return fatal errors
	//		1 non fatal error
	//		2-5 progressivly more detailed info
	int CreateClient(ExportLogMsg Log = NULL, void* p = NULL, int NewMsgVerbosity = 0);

	// Called to de-initialize the underlying speech port
	int DestroyClient();

	// Length is sound data length in bytes.
	int LoadVoiceChannel(int VoiceChannel, void* M, int Length,
                         SOUND_FORMAT Format = ULAW_8KHZ);

                                              //LV_DECODE_BLOCK | LV_DECODE_USE_OOV
    int Decode(int VoiceChannel,int GrammarSet,unsigned int flags = 0);
    
	float MeasurePitch(int VoiceChannel);
	float MeasureEnergy(int VoiceChannel);

	int DTMFDecode(const char* dtmf_str, unsigned int flags);
    int InterpretText(const char* text, unsigned flags);


	int GetNumberOfNBestAlternatives(int VoiceChannel);
	int SwitchToNBestAlternative(int VoiceChannel, int index);


	//This will return a pointer to the VoiceChannel's sound data.
	//This is always in PCM 16,000 kHz, any sound data sent to channel 
	//is converted into this format before decoding.
	//Samples is filled with number of samples of sound data.
	int	GetVoiceChannelData(int VoiceChannel, short** PCM, unsigned int* Samples);

	int WaitForEngineToIdle(int MillisecondsToWait, int VoiceChannel = -1);

	int WaitForDecode(int VoiceChannel = -1);
	
	//Returns a string explaining error code.
    static const char* ReturnErrorString(int ReturnCode);
	
	//Sets property values
    int SetPropertyEx(int property, int valuetype, void *pvalue, int target = PROP_EX_TARGET_PORT, int ndx = 0);
	static int SetClientPropertyEx(int property, int valuetype, void* pvalue);

	// Query properties
	int GetIntPropertyEx(int property, int target, int &pvalue);
	int GetFloatPropertyEx(int property, int target, float &pvalue);
	int GetStringPropertyEx(int property, int target, char *value_str, int value_len_bytes, int &actual_value_len);
	static int GetClientIntPropertyEx(int property, int &pvalue);
	static int GetClientFloatPropertyEx(int property, float &pvalue);
	static int GetClientStringPropertyEx(int property, char *value_str, int value_len_bytes, int &actual_value_len);

    static void RegisterAppLogMsg(ExportLogMsg Log, void* p = NULL, int NewMsgVerbosity = 0);
 
    static void Startup(); // Properly initialize the SpeechPort library.
    static void Shutdown(); // Cleanly un-initialize the SpeechPort library.

    HPORT GetHPort() const;

	int StreamStart();
	int StreamSendData(void* SoundData, int SoundDataLength);
	int StreamGetStatus();
	int StreamGetLength();
	int StreamSetStateChangeCallBack(LV_SRE_StreamStateChangeFn* fn, void* UserData);
	int StreamStop();
	int StreamCancel();
	int StreamSetParameter(int StreamParameter, unsigned int  StreamParameterValue);
	int StreamGetParameter(int StreamParameter, unsigned int* StreamParameterValue);
	int StreamSetParameterToDefault(int StreamParameter);

	// Load a grammar from a location. The grammars name can be an alphanumeric string,
	//a uri string (not necessarily a valid location).
	int LoadGrammar(const char* label, const char* uri);
	int LoadGrammar(const char* label_is_uri);  // Label and uri are the same

	// Load a grammar from a null terminated string. The grammars name can be an alphanumeric string,
	//a uri string (not necessarily a valid location).
	int LoadGrammarFromBuffer(const char* label, const char* buffer_string);

	int LoadGrammarFromObject(const char* label, LVGrammar& Grammar);

	// Use when LoadGrammar returns LV_GRAMMAR_SYNTAX_ERROR or LV_GRAMMAR_SYNTAX_WARNING to
	// get a description of the error.
	const char* ReturnGrammarErrorString(const char* GrammarLabel);

	bool IsGrammarLoaded(const char* label);

	// Unload the grammar by its name.
	int UnloadGrammar(const char* label);

	// Unload all grammars
	int UnloadGrammars();

	// Application-Level grammars, usable by all speechports.
	static int LoadGlobalGrammar(const char* label, const char* uri);
	static int LoadGlobalGrammar(const char* label_is_uri);
	static int LoadGlobalGrammarFromBuffer(const char* label, const char* buffer_string);
	static int LoadGlobalGrammarFromObject(const char* label, LVGrammar& Grammar);
	static int UnloadGlobalGrammar(const char* uri);

	// Get the vocabulary size for a loaded grammar
	int GetGrammarVocabSize(const char* name);

	// Activate a grammar for a decode. Multiple grammars can be active at once. They can come from the
	// current port's pool of grammars, or the global collection.
	int ActivateGrammar(const char* name);

	// Only needed if there is a name conflict between your port grammars and the 
	// global grammars, and you wish to activate a global. Otherwise just use ActivateGrammar()
	int ActivateGlobalGrammar(const char* name);

	// Deactivate a grammar
	int DeactivateGrammar(const char* name);

	// Get return values from the decode
	int GetNumberOfParses(int voice_channel);

	const char* GetParseTreeString(int voice_channel, int index);
	LVParseTree GetParseTree(int voice_channel, int index);

	int GetNumberOfInterpretations(int voice_channel);
	LVInterpretation GetInterpretation(int voice_channel, int index);
	const char* GetInterpretationString(int voice_channel, int index);

	static int GetPhoneticPronunciationCount(const char *words, const char *lang);
	static int GetPhoneticPronunciation(const char *words, const char *lang, int idx, char *pronunciation, int len);

	static int GetAvailableLanguageCount();
	static int GetAvailableLanguageIndex(int index, char *language, int length);

	int GetCallGuid(char* CallGuid, int CallGuidLength);
	int SetCustomCallGuid(const char* CallGuid);
	int GetLogFileName(int voicechannel, char *FileName, int* FileNameStringSize);

	int GetDecodeAcousticModel(int vc, char *pzmodel, int buffersize);

	int StreamSendPromptData(void* SoundData, int SoundDataLength);
	
	//Begin DEPRECATED: These function calls are deprecated in favor of using GetInterpretation
	LVSemanticData	GetInterpretationData(int voice_channel, int index);
	const char* GetInterpretationPhonemes(int voice_channel, int index);
	const char* GetInterpretationGrammarLabel(int voice_channel, int index);
	const char* GetInterpretationLanguage(int voice_channel, int index);
	const char* GetInterpretationMode(int voice_channel, int index);
	const char* GetInterpretationTopLevelRule(int voice_channel, int index);
	const char* GetInterpretationInputString(int voice_channel, int index);
	int GetInterpretationScore(int voice_channel, int index);
	//End DEPRECATED

    // Helper class, instantiate at top of "main"
	class TInitializer
	{
	public:
		TInitializer() {Startup();}
		~TInitializer() {Shutdown();}
	};

	int GetLastDecodeError(int voice_channel);

	int GetOpenPortStatus() { return OpenPortErrorCode; }

	int	DeactivateGrammars();

	static unsigned int GetAvailableLicensesCount(const char *license_type);

	static bool IsServerAvailable();

protected:
	int OpenPortErrorCode;	

	volatile HPORT HPort;


DEPRECATED_LVSPEECHPORT_METHODS
};


//*********************************************************************
//********************** inlines **************************************
//*********************************************************************
inline
LVSpeechPort::LVSpeechPort()
:HPort(NULL)
{
	OpenPortErrorCode = LV_OPEN_PORT_FAILED__OPEN_PORT_NOT_CALLED_YET;
}
//*********************************************************************
inline
//virtual 
LVSpeechPort::~LVSpeechPort()
{
	ClosePort();
}

//*********************************************************************
inline
int LVSpeechPort::AddEvent(const char* eventName)
{
	return AddEventEx(eventName, NULL);
}
inline
int LVSpeechPort::AddEventEx(const char* eventName, const char* eventData)
{
	return LV_SRE_AddEventEx(HPort, eventName, eventData);
}
//*********************************************************************
inline
int LVSpeechPort::CreateClient(ExportLogMsg Log, void* p, int NewMsgVerbosity)
{
	if (!HPort)
	{
		HPort = LV_SRE_CreateClient(&OpenPortErrorCode, Log,p,NewMsgVerbosity);
	}
	else return LV_SYSTEM_ERROR;
	return HPort ? LV_SUCCESS : LV_FAILURE;
}
//*********************************************************************
inline
int LVSpeechPort::DestroyClient() 
{
	HPORT local_hport = HPort;
	HPort = NULL;	//moved this above the call to destroy the port so that other threads will not use it while being destroyed
	int rc = LV_SRE_DestroyClient(local_hport);
	return rc;
}
//*********************************************************************
inline
int LVSpeechPort::LoadVoiceChannel(int VoiceChannel, void* M, int Length,
                                   SOUND_FORMAT Format)
{
	return LV_SRE_LoadVoiceChannel(HPort,VoiceChannel,M,Length,Format);
}
//*********************************************************************
inline 
int LVSpeechPort::Decode(int VoiceChannel,int GrammarSet,unsigned int flags)
{
	return LV_SRE_Decode(HPort,VoiceChannel,GrammarSet,flags);
}
//*********************************************************************
inline 
float LVSpeechPort::MeasurePitch(int VoiceChannel)
{
	return LV_SRE_DecodePitch(HPort,VoiceChannel);
}
//*********************************************************************
inline 
float LVSpeechPort::MeasureEnergy(int VoiceChannel)
{
	return LV_SRE_DecodeEnergy(HPort,VoiceChannel);
}
//*********************************************************************
inline
int LVSpeechPort::DTMFDecode(const char* dtmf_str, unsigned int flags)
{
	return LV_SRE_DTMFDecode(HPort, dtmf_str, flags);
}
//*********************************************************************
inline
int LVSpeechPort::InterpretText(const char* text, unsigned flags)
{
	return LV_SRE_InterpretText(HPort, text, flags);
}
//*********************************************************************
inline
int	LVSpeechPort::GetVoiceChannelData(int VoiceChannel, short** PCM,  unsigned int* Samples)
{
	return LV_SRE_GetVoiceChannelData(HPort,VoiceChannel,PCM,Samples);
}
//********************************************************************
inline
int LVSpeechPort::WaitForEngineToIdle(int MillisecondsToWait,int voicechannel)
{
	return LV_SRE_WaitForEngineToIdle(HPort,voicechannel, MillisecondsToWait);
}
//*********************************************************************
inline
int LVSpeechPort::WaitForDecode(int VoiceChannel)
{
	return LV_SRE_WaitForDecode(HPort, VoiceChannel);
}
//*********************************************************************
inline	
//static 
const char* LVSpeechPort::ReturnErrorString(int ReturnCode)
{
	return LV_SRE_ReturnErrorString(ReturnCode);
}
//*********************************************************************
inline
const char* LVSpeechPort::ReturnGrammarErrorString(const char* GrammarLabel)
{
	return LV_SRE_ReturnGrammarErrorString(HPort, GrammarLabel);
}
//*********************************************************************
inline
int LVSpeechPort::SetPropertyEx(int property,int valuetype,void *pvalue,int target,int ndx)
{
	return LV_SRE_SetPropertyEx(HPort,property,valuetype,pvalue,target,ndx);
}
inline
int LVSpeechPort::SetClientPropertyEx(int property,int valuetype,void* pvalue)
{
	return LV_SRE_SetPropertyEx(NULL,property,valuetype,pvalue,PROP_EX_TARGET_CLIENT,0);
}
//*********************************************************************
inline
int LVSpeechPort::GetIntPropertyEx(int property, int target, int &pvalue)
{
	return LV_SRE_GetIntPropertyEx(HPort, property, target, &pvalue);
}
inline
int LVSpeechPort::GetFloatPropertyEx(int property, int target, float &pvalue)
{
	return LV_SRE_GetFloatPropertyEx(HPort, property, target, &pvalue);
}
inline
int LVSpeechPort::GetStringPropertyEx(int property, int target, char *value_str, int value_len_bytes, int &actual_value_len)
{
	return LV_SRE_GetStringPropertyEx(HPort, property, target, value_str, value_len_bytes, &actual_value_len);
}
inline
// static
int LVSpeechPort::GetClientIntPropertyEx(int property, int &pvalue)
{
	return LV_SRE_GetIntPropertyEx(NULL, property, PROP_EX_TARGET_CLIENT, &pvalue);
}
inline
// static
int LVSpeechPort::GetClientFloatPropertyEx(int property, float &pvalue)
{
	return LV_SRE_GetFloatPropertyEx(NULL, property, PROP_EX_TARGET_CLIENT, &pvalue);
}
inline
// static
int LVSpeechPort::GetClientStringPropertyEx(int property, char *value_str, int value_len_bytes, int &actual_value_len)
{
	return LV_SRE_GetStringPropertyEx(NULL, property, PROP_EX_TARGET_CLIENT, value_str, value_len_bytes, &actual_value_len);
}
//*********************************************************************
inline
//static 
void LVSpeechPort::RegisterAppLogMsg(ExportLogMsg Log,void* p,int NewMsgVerbosity)
{
	LV_SRE_RegisterAppLogMsg(Log,p,NewMsgVerbosity);
}
//*********************************************************************
inline
//static
void LVSpeechPort::Startup()
{
	LV_SRE_Startup();
}
//*********************************************************************
inline
//static
void LVSpeechPort::Shutdown()
{
	LV_SRE_Shutdown();
}
//*********************************************************************
inline
HPORT LVSpeechPort::GetHPort() const 
{
	return HPort;
}
//*********************************************************************
inline
int LVSpeechPort::AddFieldToRequest(const char* KeyName, const char* String)
{
	return LV_SRE_AddFieldToRequest(GetHPort(), KeyName, String);
}
//*********************************************************************
inline
int LVSpeechPort::ClearAllFieldsToRequest()
{
	return LV_SRE_ClearAllFieldsToRequest(GetHPort());
}

//*********************************************************************
inline
int LVSpeechPort::GetDecodeMS(int voicechannel)
{
	return LV_SRE_GetDecodeMS(GetHPort(), voicechannel);
}

//*********************************************************************
inline
int LVSpeechPort::GetTotalDecodeMS(int voicechannel)
{
	return LV_SRE_GetTotalDecodeMS(GetHPort(), voicechannel);
}

//*********************************************************************
inline
int LVSpeechPort::StreamStart()
{
	return LV_SRE_StreamStart(GetHPort());
}
//*********************************************************************
inline
int LVSpeechPort::StreamSendData(void* SoundData, int SoundDataLength)
{
	return LV_SRE_StreamSendData(GetHPort(), SoundData, SoundDataLength);
}
//*********************************************************************
inline
int LVSpeechPort::StreamGetStatus()
{
	return LV_SRE_StreamGetStatus(GetHPort());
}
//*********************************************************************
inline
int LVSpeechPort::StreamGetLength()
{
	return LV_SRE_StreamGetLength(GetHPort());
}
//*********************************************************************
inline
int LVSpeechPort::StreamSetStateChangeCallBack(LV_SRE_StreamStateChangeFn* fn, void* UserData)
{
	return LV_SRE_StreamSetStateChangeCallBack(GetHPort(), fn, UserData);
}
//*********************************************************************
inline
int LVSpeechPort::StreamStop()
{
	return LV_SRE_StreamStop(GetHPort());
}
//*********************************************************************
inline
int LVSpeechPort::StreamCancel()
{
	return LV_SRE_StreamCancel(GetHPort());
}
//*********************************************************************
inline
int LVSpeechPort::StreamSetParameter(int StreamParameter, unsigned int  StreamParameterValue)
{
	return LV_SRE_StreamSetParameter(GetHPort(), StreamParameter, StreamParameterValue);
}
//*********************************************************************
inline
int LVSpeechPort::StreamGetParameter(int StreamParameter, unsigned int* StreamParameterValue)
{
	return LV_SRE_StreamGetParameter(GetHPort(), StreamParameter, StreamParameterValue);
}
//*********************************************************************
inline
int LVSpeechPort::StreamSetParameterToDefault(int StreamParameter)
{
	return LV_SRE_StreamSetParameterToDefault(GetHPort(), StreamParameter);
}
//*********************************************************************
inline
int LVSpeechPort::LoadGrammar(const char* label, const char* uri)
{
	return LV_SRE_LoadGrammar(GetHPort(), label, uri);
}
//*********************************************************************
inline
int LVSpeechPort::LoadGrammar(const char* label_is_uri)
{
	return LV_SRE_LoadGrammar(GetHPort(), label_is_uri, label_is_uri);
}
//*********************************************************************
inline
int LVSpeechPort::LoadGrammarFromBuffer(const char* label, const char* buffer_string)
{
	return LV_SRE_LoadGrammarFromBuffer(GetHPort(),label,buffer_string);
}
//*********************************************************************
inline
int LVSpeechPort::LoadGrammarFromObject(const char* label, LVGrammar& Grammar)
{
	return LV_SRE_LoadGrammarFromObject(GetHPort(),label,Grammar.GetHGrammar());
}
//*********************************************************************
inline
bool LVSpeechPort::IsGrammarLoaded(const char* label)
{
	return LV_SRE_IsGrammarLoaded(GetHPort(),label) == 1;
}
//*********************************************************************
inline
int LVSpeechPort::UnloadGrammar(const char* label)
{
	return LV_SRE_UnloadGrammar(GetHPort(),label);
}
//*********************************************************************
inline
int LVSpeechPort::UnloadGrammars()
{
	return LV_SRE_UnloadGrammars(GetHPort());
}
//*********************************************************************
//static
inline
int LVSpeechPort::LoadGlobalGrammar(const char* label, const char* uri)
{
	return LV_SRE_LoadGlobalGrammar(label,uri);
}
//*********************************************************************
//static
inline
int LVSpeechPort::LoadGlobalGrammar(const char* label_is_uri)
{
	return LV_SRE_LoadGlobalGrammar(label_is_uri,label_is_uri);
}
//*********************************************************************
//static
inline
int LVSpeechPort::LoadGlobalGrammarFromBuffer(const char* label, const char* buffer_string)
{
	return LV_SRE_LoadGlobalGrammarFromBuffer(label,buffer_string);
}
//*********************************************************************
//static
inline
int LVSpeechPort::LoadGlobalGrammarFromObject(const char* label, LVGrammar& Grammar)
{
	return LV_SRE_LoadGlobalGrammarFromObject(label,Grammar.GetHGrammar());
}
//*********************************************************************
//static
inline
int LVSpeechPort::UnloadGlobalGrammar(const char* name)
{
	//std::cerr << "LVSpeechPort::UnloadGlobalGrammars - This functionality has been removed." << std::endl;
	return LV_SRE_UnloadGlobalGrammar(name);
}
//*********************************************************************
inline
int LVSpeechPort::GetGrammarVocabSize(const char* name)
{
	return LV_SRE_ReturnGrammarVocabSize(GetHPort(),name);
}
//*********************************************************************
inline
int LVSpeechPort::ActivateGrammar(const char* name)
{
	return LV_SRE_ActivateGrammar(GetHPort(),name);
}
//*********************************************************************
inline
int	LVSpeechPort::ActivateGlobalGrammar(const char* name)
{
	return LV_SRE_ActivateGlobalGrammar(GetHPort(),name);
}
//*********************************************************************
inline
int	LVSpeechPort::DeactivateGrammar(const char* name)
{
	return LV_SRE_DeactivateGrammar(GetHPort(),name);
}
//*********************************************************************
inline
int LVSpeechPort::GetNumberOfParses(int voice_channel)
{
	return LV_SRE_GetNumberOfParses(GetHPort(),voice_channel);
}
//*********************************************************************
inline
const char* LVSpeechPort::GetParseTreeString(int voice_channel, int index)
{
	return LV_SRE_GetParseTreeString(GetHPort(),voice_channel, index);
}
//*********************************************************************
inline 
LVParseTree LVSpeechPort::GetParseTree(int voice_channel, int index)
{
	return LVParseTree(LV_SRE_CreateParseTree(GetHPort(),voice_channel, index));
}
//*********************************************************************
inline
int LVSpeechPort::GetNumberOfInterpretations(int voice_channel)
{
	return LV_SRE_GetNumberOfInterpretations(GetHPort(), voice_channel);
}
//*********************************************************************
inline
const char* LVSpeechPort::GetInterpretationString(int voice_channel,int index)
{
	return LV_SRE_GetInterpretationString(GetHPort(),voice_channel, index);
}
//*********************************************************************
inline
LVInterpretation LVSpeechPort::GetInterpretation(int voice_channel, int index)
{
	return LVInterpretation(LV_SRE_CreateInterpretation(GetHPort(), voice_channel,index));
}
//*********************************************************************
inline
LVSemanticData LVSpeechPort::GetInterpretationData(int voice_channel, int index)
{
	LVSemanticData L;
	L.data = LV_SRE_GetInterpretationData(GetHPort(), voice_channel, index);
	return L;
}
//*********************************************************************
inline
const char* LVSpeechPort::GetInterpretationPhonemes(int voice_channel, int index)
{
	return LV_SRE_GetInterpretationPhonemes(GetHPort(), voice_channel, index);
}
//*********************************************************************
inline
const char* LVSpeechPort::GetInterpretationGrammarLabel(int voice_channel, int index)
{
	return LV_SRE_GetInterpretationGrammarLabel(GetHPort(), voice_channel, index);
}
//*********************************************************************
inline
const char* LVSpeechPort::GetInterpretationLanguage(int voice_channel, int index)
{
	return LV_SRE_GetInterpretationLanguage(GetHPort(), voice_channel, index);
}
//*********************************************************************
inline
const char* LVSpeechPort::GetInterpretationMode(int voice_channel, int index)
{
	return LV_SRE_GetInterpretationMode(GetHPort(), voice_channel, index);
}
//*********************************************************************
inline
const char* LVSpeechPort::GetInterpretationTopLevelRule(int voice_channel, int index)
{
	return LV_SRE_GetInterpretationTopLevelRule(GetHPort(), voice_channel, index);
}
//*********************************************************************
inline
const char* LVSpeechPort::GetInterpretationInputString(int voice_channel, int index)
{
	return LV_SRE_GetInterpretationInputString(GetHPort(), voice_channel, index);
}

//*********************************************************************
inline
int LVSpeechPort::GetInterpretationScore(int voice_channel, int index)
{
	return LV_SRE_GetInterpretationScore(GetHPort(), voice_channel, index);
}
//*********************************************************************
inline
int LVSpeechPort::GetNumberOfNBestAlternatives(int voice_channel)
{
	return LV_SRE_GetNumberOfNBestAlternatives(GetHPort(), voice_channel);
}
//*********************************************************************
inline
int LVSpeechPort::SwitchToNBestAlternative(int voice_channel, int index)
{
	return LV_SRE_SwitchToNBestAlternative(GetHPort(),voice_channel,index);
}
//*********************************************************************
inline
int LVSpeechPort::GetLastDecodeError(int voice_channel)
{
	return LV_SRE_GetLastDecodeError(GetHPort(),voice_channel);
}
//*********************************************************************
inline
int	LVSpeechPort::DeactivateGrammars()
{
	return LV_SRE_DeactivateGrammars(GetHPort());
}
//*********************************************************************
inline
//static
unsigned int LVSpeechPort::GetAvailableLicensesCount(const char *license_type)
{
	return LV_SRE_GetAvailableLicensesCount(license_type);
}
//*********************************************************************
inline
//static
bool LVSpeechPort::IsServerAvailable()
{
	return LV_SRE_IsServerAvailable() ? true : false;
}
//*********************************************************************
// static
inline
int LVSpeechPort::GetPhoneticPronunciationCount(const char *words, const char *lang)
{
	return LV_SRE_GetPhoneticPronunciationCount(words, lang);
}
//*********************************************************************
// static
inline
int LVSpeechPort::GetPhoneticPronunciation(const char *words, const char *lang, int idx, char *pronunciation, int len)
{
	return LV_SRE_GetPhoneticPronunciation(words, lang, idx, pronunciation, len);
}
//*********************************************************************
//static
inline
int LVSpeechPort::GetAvailableLanguageCount()
{
	return LV_SRE_GetAvailableLanguageCount();
}
//*********************************************************************
//static
inline
int LVSpeechPort::GetAvailableLanguageIndex(int index, char *language, int length)
{
	return LV_SRE_GetAvailableLanguageIndex(index, language, length);
}
//*********************************************************************
inline
int LVSpeechPort::GetCallGuid(char* CallGuid, int CallGuidLength)
{
	return LV_SRE_GetCallGuid(GetHPort(), CallGuid, CallGuidLength);
}
//*********************************************************************
inline
int LVSpeechPort::GetDecodeAcousticModel(int vc,char *pzmodel,int buffersize)
{
	return LV_SRE_GetDecodeAcousticModel(GetHPort(), vc, pzmodel, buffersize);
}
//*********************************************************************
inline
int LVSpeechPort::GetLogFileName(int voicechannel, char *FileName,int* FileNameStringSize)
{
	return LV_SRE_GetLogFileName(GetHPort(), voicechannel, FileName, FileNameStringSize);
}
//*********************************************************************
inline
int LVSpeechPort::SetCustomCallGuid(const char* CallGuid)
{
	return LV_SRE_SetCustomCallGuid(GetHPort(), CallGuid);
}
//*********************************************************************
inline
int LVSpeechPort::StreamSendPromptData(void* SoundData, int SoundDataLength)
{
	return LV_SRE_StreamSendPromptData(GetHPort(), SoundData, SoundDataLength);
}

#ifndef IGNORE_LV_DEPRECATED
//*********************************************************************
inline
int LVSpeechPort::OpenPort(ExportLogMsg Log, void* p, int NewMsgVerbosity)
{
	return CreateClient(Log, p, NewMsgVerbosity);
}
//*********************************************************************
inline
int LVSpeechPort::ClosePort() 
{
	return DestroyClient();
}
// Since this method could lead to potential issues, it
// now does nothing in the current implementation.
inline
void LVSpeechPort::SetHPort(HPORT port)
{;}

#endif	// #ifndef IGNORE_LV_DEPRECATED


#endif
