#ifndef __LVTTSCLIENT_H_153C067C_FDBA_458D_BBBE_1B7534E0EA6E__INCLUDED__
#define __LVTTSCLIENT_H_153C067C_FDBA_458D_BBBE_1B7534E0EA6E__INCLUDED__

#include <LV_TTS.h>


// ============================================================================
//
// This class is a user-side light-weight wrapper around the "C" API for TTS.
// You can bypass this and use the "C" API directly.
//
// ============================================================================



class LVTTSClient
{
public:
	// Constructor
	LVTTSClient();

	// Destructor
	~LVTTSClient();

	// Create a TTS client object. Necessary to perform any TTS operations.
	LV_TTS_RETURN_CODE Initialize(const char *lang_code, const char *gender, const char *speaker_name, unsigned int sampling_rate);

	// Destroys the specified client object, releasing its memory
	LV_TTS_RETURN_CODE Destroy();

	// Perform the actual text to speech operation for the given sentence or SSML text.
	LV_TTS_RETURN_CODE Synthesize(const char *input_sentence, unsigned int flags);

	// Wait for a response back from the TTS server or a timeout event before returning.
	LV_TTS_RETURN_CODE WaitForSynthesis(unsigned int timeout_ms);

	// Get the size of the synthesized audio buffer that was obtained from a Synthesize call.
	LV_TTS_RETURN_CODE GetSynthesizedAudioBufferLength(int * buffer_length);

	// Read parts of the synthesized audio buffer that was obtained from a Synthesize call.
	LV_TTS_RETURN_CODE GetSynthesizedAudioBuffer(void *buffer, int buffer_size_in_bytes, int * bytes_returned);

	// Reset the buffer read offset so that the next invocation of GetSynthesizedAudioBuffer starts
	// from the first sample in the buffer.
	LV_TTS_RETURN_CODE RewindBuffer();

	// Returns the sound format of the synthesized audio.
	LV_TTS_RETURN_CODE GetSynthesizedAudioFormat(int * audio_format);

	// Returns the sampling rate of the synthesized audio.
	LV_TTS_RETURN_CODE GetSynthesizedAudioSampleRate(int * sample_rate);

	// Returns the corresponding number of sentence marks available in the synthesized response.
	LV_TTS_RETURN_CODE GetSentenceMarksCount(int * return_count);

	// Returns the corresponding number of word marks available in the synthesized response.
	LV_TTS_RETURN_CODE GetWordMarksCount(int * return_count);

	// Returns the corresponding number of SSML Speech-Marker marks available in the synthesized response.
	LV_TTS_RETURN_CODE GetSSMLMarksCount(int * return_count);

	// Move the buffer offset to the beginning of the specified sentence so that the next invocation of
	// GetSynthesizedAudioBuffer returns samples starting at that offset.
	LV_TTS_RETURN_CODE JumpToSentence(unsigned int sentence_idx);

	// Move the buffer offset to the beginning of the specified word so that the next invocation of
	// GetSynthesizedAudioBuffer returns samples starting at that offset.
	LV_TTS_RETURN_CODE JumpToWord(unsigned int word_idx);

	// Move the buffer offset to the beginning of the specified SSML Speech-Marker so that the next invocation of
	// GetSynthesizedAudioBuffer returns samples starting at that offset.
	LV_TTS_RETURN_CODE JumpToSSMLMark(unsigned int ssml_mark_idx);

	// Returns the offset of the beginning of the specified sentence in the synthesized audio buffer. 
	LV_TTS_RETURN_CODE GetSentenceOffsetInBuffer(unsigned int sentence_idx, int *offset);

	// Returns the offset of the beginning of the specified word in the synthesized audio buffer. 
	LV_TTS_RETURN_CODE GetWordOffsetInBuffer(unsigned int word_idx, int *offset);

	// Returns the offset of the beginning of the specified Speech-Marker in the synthesized audio buffer. 
	LV_TTS_RETURN_CODE GetSSMLMarkOffsetInBuffer(unsigned int ssml_mark_idx, int *offset);

	// Returns the name of the SSML mark at the specified index
	LV_TTS_RETURN_CODE GetSSMLMarkName(unsigned int ssml_mark_idx, char *name_buffer, int name_buffer_length);

	// Returns 1 if an SRE is available, otherwise 0 (not using bool since this is a C API function)
	static LV_TTS_RETURN_CODE IsServerAvailable();

	// Provides the last SSML syntax error information, if any
	LV_TTS_RETURN_CODE GetLastSSMLError(char *error_buffer, int error_buffer_length);

	// Set port level configuration properties for the LV_TTS interface
	LV_TTS_RETURN_CODE SetPropertyEx(int property, int valuetype, void *pvalue);

	// Set application level configuration properties for the LV_TTS interface
	static LV_TTS_RETURN_CODE SetClientPropertyEx(int property, int valuetype, void* pvalue);

	// Query port level configuration settings for int type options for the LV_TTS interface
	LV_TTS_RETURN_CODE GetIntPropertyEx(int property, int target, int &pvalue);

	// Query port level configuration settings for float type options for the LV_TTS interface
	LV_TTS_RETURN_CODE GetFloatPropertyEx(int property, int target, float &pvalue);

	// Query port level configuration settings for string type options for the LV_TTS interface
	LV_TTS_RETURN_CODE GetStringPropertyEx(int property, int target, char *value_str, int value_len_bytes, int &actual_value_len);

	// Query application level configuration settings for integer type options for the LV_TTS interface
	static LV_TTS_RETURN_CODE GetClientIntPropertyEx(int property, int &pvalue);

	// Query application level configuration settings for float type options for the LV_TTS interface
	static LV_TTS_RETURN_CODE GetClientFloatPropertyEx(int property, float &pvalue);

	// Query application level configuration settings for string type options for the LV_TTS interface
	static LV_TTS_RETURN_CODE GetClientStringPropertyEx(int property, char *value_str, int value_len_bytes, int &actual_value_len);

	// This function is used to set the ID used in callsre logging. If not used, a globally unique id
	// will be generated automatically
	LV_TTS_RETURN_CODE SetCustomCallGuid(const char* call_guid);

	// This function is used to get the ID used in callsre logging. It can be used in conjunction with
	// LV_TTS_SetCustomCallGuid() to link events across SRE ports and TTS ports to the same call log 
	LV_TTS_RETURN_CODE GetCallGuid(char* call_guid, int call_guid_length);

	// This function is used to add logging events to the callsre file
	LV_TTS_RETURN_CODE AddEvent(const char* event_name);

	// Adds a custom string to a named field in the call log file
	LV_TTS_RETURN_CODE AddFieldToRequest(const char * key_name, const char * string_value);

	// Get a handle to the internal HTTSCLIENT instance
	LV_TTS_RETURN_CODE GetClientHandle(HTTSCLIENT &client_handle);


private:

	// Handle to this client instance
	HTTSCLIENT m_Client;
};



// ==========================================================================
// ======================== inline functions ================================
// ==========================================================================



inline
LVTTSClient::LVTTSClient()
: m_Client(NULL)
{}


inline
LVTTSClient::~LVTTSClient()
{
	Destroy();
}


inline
LV_TTS_RETURN_CODE LVTTSClient::Initialize(const char *lang_code, const char *gender, const char *speaker_name,
										   unsigned int sampling_rate)
{
	LV_TTS_RETURN_CODE error_return_code = LV_SUCCESS;

	if(m_Client == NULL)
	{
		m_Client = LV_TTS_CreateClient(lang_code, gender, speaker_name, sampling_rate, &error_return_code);
	}

	return error_return_code;
}


inline
LV_TTS_RETURN_CODE LVTTSClient::Destroy()
{
	if(m_Client != NULL)
	{
		HTTSCLIENT ttsClient = m_Client;
		m_Client = NULL;
		return LV_TTS_DestroyClient(ttsClient);
	}

	return LV_SUCCESS;
}

inline
LV_TTS_RETURN_CODE LVTTSClient::Synthesize(const char *input_sentence, unsigned int flags)
{
	if(m_Client != NULL)
	{
		return LV_TTS_Synthesize(m_Client, input_sentence, flags);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}

inline
LV_TTS_RETURN_CODE LVTTSClient::WaitForSynthesis(unsigned int timeout_ms)
{
	if(m_Client != NULL)
	{
		return LV_TTS_WaitForSynthesis(m_Client, timeout_ms);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetSynthesizedAudioBufferLength(int * buffer_length)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetSynthesizedAudioBufferLength(m_Client, buffer_length);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetSynthesizedAudioBuffer(void *buffer, int buffer_size_in_bytes, int * bytes_returned)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetSynthesizedAudioBuffer(m_Client, buffer, buffer_size_in_bytes, bytes_returned);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::RewindBuffer()
{
	if(m_Client != NULL)
	{
		return LV_TTS_RewindBuffer(m_Client);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetSynthesizedAudioFormat(int * audio_format)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetSynthesizedAudioFormat(m_Client, audio_format);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetSynthesizedAudioSampleRate(int * sample_rate)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetSynthesizedAudioSampleRate(m_Client, sample_rate);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetSentenceMarksCount(int * return_count)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetSentenceMarksCount(m_Client, return_count);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetWordMarksCount(int * return_count)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetWordMarksCount(m_Client, return_count);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetSSMLMarksCount(int * return_count)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetSSMLMarksCount(m_Client, return_count);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::JumpToSentence(unsigned int sentence_idx)
{
	if(m_Client != NULL)
	{
		return LV_TTS_JumpToSentence(m_Client, sentence_idx);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::JumpToWord(unsigned int word_idx)
{
	if(m_Client != NULL)
	{
		return LV_TTS_JumpToWord(m_Client, word_idx);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::JumpToSSMLMark(unsigned int ssml_mark_idx)
{
	if(m_Client != NULL)
	{
		return LV_TTS_JumpToSSMLMark(m_Client, ssml_mark_idx);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetSentenceOffsetInBuffer(unsigned int sentence_idx, int *buffer_offset)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetSentenceOffsetInBuffer(m_Client, sentence_idx, buffer_offset);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetWordOffsetInBuffer(unsigned int word_idx, int *buffer_offset)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetWordOffsetInBuffer(m_Client, word_idx, buffer_offset);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetSSMLMarkOffsetInBuffer(unsigned int ssml_mark_idx, int *buffer_offset)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetSSMLMarkOffsetInBuffer(m_Client, ssml_mark_idx, buffer_offset);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetSSMLMarkName(unsigned int ssml_mark_idx, char * name_buffer, int name_buffer_length)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetSSMLMarkName(m_Client, ssml_mark_idx, name_buffer, name_buffer_length);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


// static
inline
LV_TTS_RETURN_CODE LVTTSClient::IsServerAvailable()
{
	return LV_TTS_IsServerAvailable();
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetLastSSMLError(char *error_buffer, int error_buffer_length)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetLastSSMLError(m_Client, error_buffer, error_buffer_length);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::SetPropertyEx(int property, int valuetype, void *pvalue)
{
	if(m_Client != NULL)
	{
		return LV_TTS_SetPropertyEx(m_Client, property, valuetype, pvalue, PROP_EX_TARGET_PORT);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


// static
inline
LV_TTS_RETURN_CODE LVTTSClient::SetClientPropertyEx(int property, int valuetype, void* pvalue)
{
	return LV_TTS_SetPropertyEx(NULL, property, valuetype, pvalue, PROP_EX_TARGET_CLIENT);
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetIntPropertyEx(int property, int target, int &pvalue)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetIntPropertyEx(m_Client, property, target, &pvalue);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetFloatPropertyEx(int property, int target, float &pvalue)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetFloatPropertyEx(m_Client, property, target, &pvalue);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetStringPropertyEx(int property, int target, char *value_str, int value_len_bytes, int &actual_value_len)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetStringPropertyEx(m_Client, property, target, value_str, value_len_bytes, &actual_value_len);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
// static
LV_TTS_RETURN_CODE LVTTSClient::GetClientIntPropertyEx(int property, int &pvalue)
{
	return LV_TTS_GetIntPropertyEx(NULL, property, PROP_EX_TARGET_CLIENT, &pvalue);
}


inline
// static
LV_TTS_RETURN_CODE LVTTSClient::GetClientFloatPropertyEx(int property, float &pvalue)
{
	return LV_TTS_GetFloatPropertyEx(NULL, property, PROP_EX_TARGET_CLIENT, &pvalue);
}

inline
// static
LV_TTS_RETURN_CODE LVTTSClient::GetClientStringPropertyEx(int property, char *value_str, int value_len_bytes, int &actual_value_len)
{
	return LV_TTS_GetStringPropertyEx(NULL, property, PROP_EX_TARGET_CLIENT, value_str, value_len_bytes, &actual_value_len);
}


inline
LV_TTS_RETURN_CODE LVTTSClient::SetCustomCallGuid(const char* call_guid)
{
	if(m_Client != NULL)
	{
		return LV_TTS_SetCustomCallGuid(m_Client, call_guid);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetCallGuid(char* call_guid, int call_guid_length)
{
	if(m_Client != NULL)
	{
		return LV_TTS_GetCallGuid(m_Client, call_guid, call_guid_length);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::AddEvent(const char* event_name)
{
	if(m_Client != NULL)
	{
		return LV_TTS_AddEvent(m_Client, event_name);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::AddFieldToRequest(const char * key_name, const char * string_value)
{
	if(m_Client != NULL)
	{
		return LV_TTS_AddFieldToRequest(m_Client, key_name, string_value);
	}
	else
	{
		return LV_SYSTEM_ERROR;
	}
}


inline
LV_TTS_RETURN_CODE LVTTSClient::GetClientHandle(HTTSCLIENT &client_handle)
{
	if(m_Client != NULL)
	{
		client_handle = m_Client;
		return LV_SUCCESS;
	}
	else
	{
		return LV_FAILURE;
	}
}



#endif	/* __LVTTSCLIENT_H_153C067C_FDBA_458D_BBBE_1B7534E0EA6E__INCLUDED__ */
