#if !defined(LV_SRE_h)
#define LV_SRE_h

#include <LV_SRE_Defines.h>
#include "LV_SRE_Deprecated.h"
#include "LV_SRE_Advanced.h"
#include "LV_SRE_ParseTree.h"
#include "LV_SRE_Semantic.h"
#include "LV_SRE_Grammar.h"

//*********************************************************************
//prevent name-mangling
#if defined(__cplusplus)
extern "C"
{
#endif

/*****************************************************************
* Startup and shutdown routines
*
* Overview:
* The LV_SRE_CreateClient function is called to obtain a handle to an SRE client port(HPORT), 
* which is requrired for any SRE functionality
* LV_SRE_DestroyClient is used to release the opened port handle.
*/

/* LV_SRE_CreateClient -- obtains an SRE client port handle, optionally registering a logging callback.
 int* error_return_code : Pointer to a value, which can be used with LV_SRE_ReturnErrorString
                         to obtain a message indicating why the call failed (if it did fail)	
 ExportLogMsg log_callback : Pointer to a user defined function which will receive logging information
 void* log_callback_user_data : Pointer to user defind data will be passed into the ExportLogMsg function
 int log_verbosity : Should be a value between 0 - 6. 0 - minimal logging; 6 - maximum logging
 Return Values - NULL: Speechport error; non-NULL: success */
LV_SRE_API HPORT LV_SRE_CreateClient(int* error_return_code, ExportLogMsg log_callback, void *log_callback_user_data, int log_verbosity);

/* LV_SRE_DestroyClient -- releases previously opened port handle
 HPORT hport : the port handle to be released, obtained from a previous call to LV_SRE_CreateClient
 Return Value: LV_SUCCESS No errors, the port has successfully shut down; LV_INVALID_HPORT The port was unable to shutdown*/ 
LV_SRE_API int LV_SRE_DestroyClient(HPORT hport);

/* 
 * End startup and shutdown routines
 *****************************************************************/

/*****************************************************************
 * Setup/configuration routines
 *
 * Overview:
 * Once a port is opened, you may want to set one or more optional
 * configurations before using it to make further calls to the API.
 * Without these calls, the SRE will default to the configuration
 * settings provided in the client_property.conf file.
 * You may also register optional callback functions to handle 
 * certain events from the SRE
 */

/* LV_SRE_SetPropertyEx -- sets configuration properties for the LV_SRE interface
 HPORT hport: the handle to the port obtained from OpenPort
 int property: one of several enumerated properties, described below
 int valuetype: type of following parameter. legal values are:
			PROP_EX_VALUE_TYPE_INT,PROP_EX_VALUE_TYPE_INT_PTR,
			PROP_EX_VALUE_TYPE_STRING,PROP_EX_VALUE_TYPE_FLOAT_PTR 
 void* pvalue: the value or a pointer to the value to be set for the property
 int target: the target scope of the property. legal values are:
			PROP_EX_TARGET_PORT (local), PROP_EX_TARGET_CLIENT (global)
			
 Properties 
	PROP_EX_SRE_SERVERS: a string containg the IP address and port of the SRE servers to direct requests to
		e.g. "127.0.0.1;10.0.0.1:5721" specifies two servers: 127.0.0.1 (port 5730) and 10.0.0.1 (port 5721)
	PROP_EX_LICENSE_SERVERS: A string containg the IP address and port of the License Servers to direct license requests to
		e.g. "127.0.0.1;10.0.0.1:5721" specifies two servers: 127.0.0.1 (port 7569) and 10.0.0.1 port 5721
	PROP_EX_TRIM_SILENCE_VALUE: Sets how aggressively silence is trimmed from the audio during pre-processing. Values range
		from 0 to 1000; lower values corresponding to more aggressive trimming. Default value: 970
	PROP_EX_NOISE_REDUCTION_ENABLE: an enumerated type that specifies what type of noise reduction algorithm is
		applied to audio. NOISE_REDUCTION_DEFAULT is recommended, but other settings may produce better results
		Possible values: NOISE_REDUCTION_NONE, NOISE_REDUCTION_DEFAULT, NOISE_REDUCTION_ALTERNATE,NOISE_REDUCTION_ADAPTIVE
	PROP_EX_MAX_NBEST_RETURNED: an integer that specifies the maximum number of n-best results to be returned by the SRE 
	PROP_EX_DECODE_TIMEOUT: In a non-blocking decode, this is the timeout value, in milliseconds, used by LV_SRE_WaitForDecode
		and LVSpeechPort::WaitForDecode functions. In blocking decode, this is the time to wait until the decode times out
		and returns an error from LV_SRE_Decode and LVSpeechPort::Decode.
	PROP_EX_LOAD_GRAMMAR_TIMEOUT: an integer that specifies the number of milliseconds to wait for a response to a load
		grammar request.
	PROP_EX_ACOUSTIC_MODEL_RESOLUTION: An enumerated type that specifies the resolution of acoustic model to be used in decode.
		Higher resolution models may improve accuracy at the cost of speed. Possible Values:
		PROP_EX_LOW_RESOLUTION_ACOUSTIC_MODEL, PROP_EX_MIDDLE_RESOLUTION_ACOUSTIC_MODEL, PROP_EX_HIGH_RESOLUTION_ACOUSTIC_MODEL
	PROP_EX_SPEED_VS_ACCURACY: an integer between 0-100 specifiying how much accuracy is valued from an SRE decode,
		at the cost of speed. Lower values correspond to faster decodes; 100 specifies maximum accuracy. 
		PROP_EX_DECODE_TIMEOUT: an integer that specifies the number of milliseconds to wait for a response to a decode request.
		This could be the amount of time before returning from LV_SRE_Decode if blocking or LV_SRE_WaitForDecode
	PROP_EX_STRICT_SISR_COMPLIANCE: 0 - tag-format <semantics/1.0> specifies an older draft of SISR. use <semantics/1.0.2006>
		for final draft. 1 - tag-format <semantics/1.0> specifies the final draft SISR
	PROP_EX_BUILTIN_GRAMMAR_LANGUAGE: a string that specifies the language that builtin grammars default to. Currently, only
		"en-US" and "es-MX" are supported.
	PROP_EX_LICENSE_TYPE: a string specfying the license type to be used when opening a port
		Possible values: PROP_EX_LICENSE_VOXLITE, PROP_EX_LICENSE_SPEECHPORT, PROP_EX_LICENSE_AMD, PROP_EX_LICENSE_SLM
	PROP_SAVE_SOUND_FILES: determines how much decode information is saved to the Responses directory on disk
		These are .callsre files used by the Speech Tuner to review the contents of interactions.
		Possible Values: SAVE_SOUND_FILES_NONE, SAVE_SOUND_FILES_BASIC, SAVE_SOUND_FILES_ADVANCED, SAVE_SOUND_FILES_ALL
		PROP_EX_DECODE_THREAD_PRIORITY: an enumerated value that sets a thread priority for decode tasks on the SRE server
		Posslbie Values: PROP_EX_DECODE_THREAD_PRIORITY_LOW, PROP_EX_DECODE_THREAD_PRIORITY_NORMAL, PROP_EX_DECODE_THREAD_PRIORITY_HIGH*/
LV_SRE_API int LV_SRE_SetPropertyEx(HPORT hport,int property,int valuetype,void *pvalue,
									int target,int ndx);

/* LV_SRE_GetStringPropertyEx -- query configuration settings for string type options for the LV_SRE interface
 HPORT hport: the handle to the port obtained from OpenPort
 int property: one of several enumerated properties of string type, described below
 int target: the target scope of the property. legal values are:
			PROP_EX_TARGET_PORT (local), PROP_EX_TARGET_CLIENT (global)
 char *value_str: buffer to store the queried value
 int value_len_bytes: size of the provided value_str buffer in bytes
 int *actual_value_len: the actual amount of buffer in value_str used to store the queried value.
            If value_str is set to NULL or value_len_bytes set to zero, no copying is performed and
			the value returned in actual_value_len indicates the total size of value_str required to
			completely store the queried value, only if LV_SUCCESS is returned.

Properties 
	PROP_EX_SRE_SERVERS: Provides list of (server:ip) pairs separated by ';'
	PROP_EX_LICENSE_SERVERS: Provides a list of license server addresses (server:ip pairs) separated by ';'
	PROP_EX_LICENSE_TYPE: A string specfying the license type the is used when opening a port
		Possible values: "Auto", "SpeechPort", "Voxlite", "AMD"
	PROP_EX_BUILTIN_GRAMMAR_LANGUAGE: a string that specifies the language that builtin grammars default to.

Possible return values: LV_SUCCESS, LV_INVALID_HPORT, LV_INVALID_PROPERTY, LV_INVALID_PROPERTY_TARGET*/
LV_SRE_API int LV_SRE_GetStringPropertyEx(HPORT hport, int property, int target,
										char *value_str, int value_len_bytes, int *actual_value_len);

/* LV_SRE_GetFloatPropertyEx -- query configuration settings for floating point type options for the LV_SRE interface
 HPORT hport: the handle to the port obtained from OpenPort
 int property: one of several enumerated properties of floating point type, described below
 int target: the target scope of the property. legal values are:
			PROP_EX_TARGET_PORT (local), PROP_EX_TARGET_CLIENT (global)
 float *pvalue: address to store the queried value at

Properties 
	Currently, no properties are supported.

Possible return values: LV_SUCCESS, LV_INVALID_HPORT, LV_INVALID_PROPERTY, LV_INVALID_PROPERTY_TARGET*/
LV_SRE_API int LV_SRE_GetFloatPropertyEx(HPORT hport, int property, int target, float *pvalue);

/* LV_SRE_GetIntPropertyEx -- query configuration settings for integer type options for the LV_SRE interface
 HPORT hport: the handle to the port obtained from OpenPort
 int property: one of several enumerated properties of integer type, described below
 int target: the target scope of the property. legal values are:
			PROP_EX_TARGET_PORT (local), PROP_EX_TARGET_CLIENT (global)
 int *pvalue: address to store the queried value at

Properties
	PROP_EX_MAX_NBEST_RETURNED: an integer that specifies the maximum number of n-best results that would be returned by the SRE
	PROP_EX_DECODE_TIMEOUT: The amount of time before returning from LV_SRE_Decode (if blocking) or LV_SRE_WaitForDecode
	PROP_EX_LOAD_GRAMMAR_TIMEOUT: The amount of time (in milliseconds) waited for the response to a load grammar request.
	PROP_SAVE_SOUND_FILES: The value used in determining how much decode information is saved to the Responses directory on disk
		These are .callsre files used by the Speech Tuner to review the contents of interactions.
		Possible return values: SAVE_SOUND_FILES_NONE, SAVE_SOUND_FILES_BASIC, SAVE_SOUND_FILES_ADVANCED, SAVE_SOUND_FILES_ALL
	PROP_EX_TRIM_SILENCE_VALUE: an integer specifiying how much silence will be removed from the audio before decode.
	PROP_EX_ACOUSTIC_MODEL_RESOLUTION: An enumerated type that specifies the resolution of acoustic model that would be used in decode. Higher resolution models
		may improve accuracy at the cost of speed.
		Possible return values: PROP_EX_LOW_RESOLUTION_ACOUSTIC_MODEL, PROP_EX_MIDDLE_RESOLUTION_ACOUSTIC_MODEL, PROP_EX_HIGH_RESOLUTION_ACOUSTIC_MODEL
	PROP_EX_SPEED_VS_ACCURACY: an integer between 0-100 specifiying how much accuracy will be valued in an SRE decode,
		at the cost of speed. Lower values correspond to faster decodes; 100 means maximum accuracy.
	PROP_EX_STRICT_SISR_COMPLIANCE: A return of 0 indicates that tag-format <semantics/1.0> specifies an older draft of SISR. <semantics/1.0.2006> must be used for final draft
		A return of 1 indicates that tag-format <semantics/1.0> specifies the final draft SISR
	PROP_EX_NOISE_REDUCTION_ENABLE: an enumerated type that indicates what type of noise reduction algorithm will be
		applied to audio.
		Possible return values: NOISE_REDUCTION_NONE, NOISE_REDUCTION_DEFAULT, NOISE_REDUCTION_ALTERNATE,NOISE_REDUCTION_ADAPTIVE

Possible return values: LV_SUCCESS, LV_INVALID_HPORT, LV_INVALID_PROPERTY, LV_INVALID_PROPERTY_TARGET*/
LV_SRE_API int LV_SRE_GetIntPropertyEx(HPORT hport, int property, int target, int *pvalue);

/* LV_SRE_StreamSetParameter -- sets configuration options for the streaming interface
 HPORT hport: the handle to the port obtained from OpenPort
 int StreamParameter: one of several enumerated properties, described below
 unsigned int SreamParameterValue: the value to be set for the property
	STREAM_PARM_SOUND_FORMAT: specifies sound data type stream handles - uses SOUND_FORMAT enum - default is ULAW_8KHZ
	STREAM_PARM_VOICE_CHANNEL: voice channel to load sound data to on stream completion
	STREAM_PARM_GRAMMAR_SET: grammar set to use with auto decode streams
	STREAM_PARM_DECODE_FLAGS: decode flags to send with auto decode streams
	STREAM_PARM_AUTO_DECODE: if true will start decode immeditaly on end-of-speech detection or a call to StopStream()
	STREAM_PARM_BARGE_IN_TIMEOUT: amount of audio that can be streamed without barge-in occuring, before the state of
		the stream changes to STREAM_STATUS_BARGE_IN_TIMEOUT
	STREAM_PARM_END_OF_SPEECH_TIMEOUT: amount of audio that can be streamed after barge-in without end-of-speech 
		triggering, before the stream state changes to STREAM_STATUS_END_SPEECH_TIMEOUT
	STREAM_PARM_DETECT_BARGE_IN: if true, audio before start-of-speech is dectected is discarded
	STREAM_PARM_DETECT_END_OF_SPEECH: if true, audio after end-of-speech is detected is discarded
	STREAM_PARM_VAD_WIND_BACK: the amount of audio before detected start-of-speech to be load into the voice channel
	STREAM_PARM_VAD_EOS_DELAY: the amount of audio after the detected end-of-speech to be loaded into voice channel
	STREAM_PARM_VAD_SNR_SENSITIVITY: Determines how much louder the speaker must be than the background noise in order to 
		trigger barge-in. The smaller this value, the easier it will be to trigger barge-in. 
	STREAM_PARM_VAD_VOLUME_SENSITIVITY: Determines the volume required to trigger barge-in. The smaller the value, the 
		more sensitive barge-in will be. Primarily used to deal with poor echo cancellation.
	STREAM_PARM_VAD_STREAM_INIT_DELAY: Sets the amount of time in ms that can be guaranteed to be silence/noise for
		VAD initialization
	STREAM_PARM_END_OF_SPEECH_DETECTION: Changes voice activity detection delay for end of speech detection based to one of four modes.
		Possible values: STREAM_END_OF_SPEECH_DETECTION_SINGLE_WORDS (500ms), STREAM_END_OF_SPEECH_DETECTION_PHRASES_NO_PAUSES (800ms),
		STREAM_END_OF_SPEECH_DETECTION_PHRASES_WITH_PAUSES (1200ms) and STREAM_END_OF_SPEECH_DETECTION_NORMAL (800ms).
	STREAM_PARM_VAD_BARGEIN_THRESHOLD: A higher value makes the VAD more sensitive to speech which means that the VAD is very sure the
		data is speech before barge in. Raising the value will reject more false positives/noises.*/
LV_SRE_API int LV_SRE_StreamSetParameter(HPORT hport, int StreamParameter, unsigned int StreamParameterValue); 

/* LV_SRE_RegisterAppLogMsg -- specifies a call-back function for global application logging events
 ExportLogMsg Log: the function to be called upon logging event
 void* p: User specified data to be passed to the callback function
 int NewMsgVerbosity: verbosity threshold for logging messages*/
LV_SRE_API void LV_SRE_RegisterAppLogMsg(ExportLogMsg Log, void* p, int NewMsgVerbosity);

/* LV_SRE_StreamSetStateChangeCallBack -- specifies a call-back function for handling stream state changes
 HPORT hport: the handle to the port obtained from OpenPort
 LV_SRE_StreamStateChangeFn* fn: the function to be called upon stream state change
 void* UserData: User specified data to be passed to the callback function*/
LV_SRE_API int LV_SRE_StreamSetStateChangeCallBack(HPORT hport, LV_SRE_StreamStateChangeFn* fn, void* UserData); 

/* End setup/configuration routines
 **************************************************/

/*****************************************************************
* Audio handling routines 
 *
 * Overview:
 * Audio must be loaded into a voice channel before it can be used by the SRE for decoding
 * There are two ways to load the audio: 1) Directly using LV_SRE_LoadVoiceChannel, or
 *										 2) Using our streaming interface
 * LV_SRE_LoadVoiceChannel assumes that the endpointing of speech has already been done 
 * The streaming interface assumes a continuous stream of audio containing both sections
 * of speech and sections of silence/non-speech. The audio stream must start with silence
 * The streaming interface also uses various configuration options, set by LV_SRE_StreamSetParameter
 *		and also a state change callback function, specified in LV_SRE_StreamSetStateChangeCallBack
 */

/* LV_SRE_LoadVoiceChannel -- Directly loads a binary audio data into a voicechannel for decoding
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient
 int VoiceChannel : the index of the voicechannel to load the audio into. Should be a value between 0-64 
 void* M : A pointer to a binary buffer of data containing audio
 int Length: the length, in bytes, of the above buffer
 SOUND_FORMAT Format: the sound format of the audio: Current supported formats are: ULAW_8KHZ, PCM_8KHZ, PCM_16KHZ,	ALAW_8KHZ */
LV_SRE_API int LV_SRE_LoadVoiceChannel(HPORT hport,int VoiceChannel,void* M,int Length,
									   enum SOUND_FORMAT Format);//,const char* SoundFileName);

/* LV_SRE_StreamStart -- Sets up a new stream. All stream parameters should be set before calling StreamStart
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient */
LV_SRE_API int LV_SRE_StreamStart(HPORT hport); 

/* LV_SRE_StreamSendData -- Sends a chunk of audio data into the streaming interface
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient
 void* SoundData : A pointer to a binary buffer of data containing audio
 int SoundDataLength: the length, in bytes, of the above buffer*/
LV_SRE_API int LV_SRE_StreamSendData(HPORT hport, void* SoundData, int SoundDataLength);

/* LV_SRE_StreamStop -- stops accepting stream data and loads sound channel(specified in parameters) with streamed data
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient */
LV_SRE_API int LV_SRE_StreamStop(HPORT hport);  

/* LV_SRE_StreamStop -- stops accepting stream data, discards stream data 
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient */
LV_SRE_API int LV_SRE_StreamCancel(HPORT hport);

/* End audio handling routines 
 **************************************************/


/*****************************************************************
/* Grammar handling routines
 *
 * Overview:
 * An SRGS grammar must be loaded into the SRE, specified by a label, before a successful decode
 * The SRGS grammar may come from one of two sources: 1) A file, specified by a URL, a local system filepath
 *													  2) A character buffer containing the grammar text
 * Once a grammar is loaded, it must be activated to be flagged for use in a decode. 
 * More than one grammar can be active in the ACTIVE_GRAMMAR_SET
 * A loaded grammar can either have a local scope, only usable within current port object
 *  or an application-level global scope, useable by all speechports in the application
 */

/* LV_SRE_LoadGrammar -- loads an SRGS from a file into the SRE, under the identifier specified by "label"
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 const char* label: a unique identifier string used to reference the grammar after it is loaded
 const char* uri: either a universal remote location for the grammar file or a local filepath */
LV_SRE_API int LV_SRE_LoadGrammar(HPORT hport, const char* label, const char* uri);
LV_SRE_API int LV_SRE_LoadGlobalGrammar(const char* label, const char* uri);

/* LV_SRE_LoadGrammarFromBuffer -- loads an SRGS from a file into the SRE, under the identifier specified by "gram"
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 const char* gram: a unique identifier string used to reference the grammar after it is loaded
 const char* buffer: a character buffer containing the text of the grammar */
LV_SRE_API int LV_SRE_LoadGrammarFromBuffer(HPORT hport, const char* gram, const char* buffer);
LV_SRE_API int LV_SRE_LoadGlobalGrammarFromBuffer(const char* label, const char* file_buffer);

/*	LV_SRE_ReturnGrammarVocabSize: Used after a LoadGrammar to return the vocabulary size
HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
const char* gram_name : the label that was specified in the call to LV_SRE_LoadGrammar*/
LV_SRE_API int LV_SRE_ReturnGrammarVocabSize(HPORT hport, const char* gram_name);

/* LV_SRE_ActivateGrammar -- marks a grammar loaded under the identifier specified by "gram_name" to be used in the next decode
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 const char* gram_name: the unique identifier string specified in LoadGrammar*/
LV_SRE_API int LV_SRE_ActivateGrammar(HPORT hport, const char* gram_name);
LV_SRE_API int LV_SRE_ActivateGlobalGrammar(HPORT hport, const char* gram_name);

/* LV_SRE_DeactivateGrammar -- unmarks a grammar loaded under the identifier specified by "gram_name" so that it is not used in the next decode
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 const char* gram_name: the unique identifier string specified in LoadGrammar*/
LV_SRE_API int LV_SRE_DeactivateGrammar(HPORT hport, const char* gram_name);

/* LV_SRE_DeactivateGrammar -- unmarks all grammars to be used in the next decode
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient */
LV_SRE_API int LV_SRE_DeactivateGrammars(HPORT hport);

/* LV_SRE_UnloadGrammar -- Removes an SRGS from a file from memory, making it unable to be referenced or activated
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 const char* name: the unique identifier string specified in LoadGrammar*/
LV_SRE_API int LV_SRE_UnloadGrammar(HPORT hport, const char* name);
LV_SRE_API int LV_SRE_UnloadGlobalGrammar(const char* grammar_name);

/* LV_SRE_UnloadGrammars -- Removes all SRGS from a file from memory, making them unable to be referenced or activated
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient */
LV_SRE_API int LV_SRE_UnloadGrammars(HPORT hport);

/* End grammar handling routines 
 **************************************************/




/*****************************************************************
 * SRE request routines
 * 
 * Overview:
 * Once a SpeechPort has been opened and audio and grammars have been loaded, it is ready to send
 * a decode request to the SRE server to perform speech recognition.
 * Alternatively, you may input a dtmf string or text string to perform a parse on the active grammar set
 *
 */

/* LV_SRE_Decode -- sends a decode request to the SRE server
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that audio was loaded into using LV_SRE_LoadVoiceChannel or LV_SRE_StreamSendData
 int GrammarSet : the GrammarSet that grammars were loaded into, probably LV_ACTIVE_GRAMMAR_SET when using SRGS grammars
 int flags : mostly unused in the current implementation besides LV_DECODE_BLOCK, 
	Setting the LV_DECODE_BLOCK bit in flags causes LV_SRE_Decode to wait for a response back from the SRE server before returning*/
LV_SRE_API int LV_SRE_Decode(HPORT hport, int VoiceChannel, int GrammarSet, unsigned int flags);

/* LV_SRE_WaitForEngineToIdle -- wait for a response back from the SRE server or a timeout event before returning
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int ms : number of milliseconds to wait before giving up waiting for a decode
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode*/
LV_SRE_API int LV_SRE_WaitForEngineToIdle(HPORT hport, int voicechannel, int ms);

/* LV_SRE_WaitForDecode -- wait for a response back from the SRE server or a decode timeout event before returning
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode*/
LV_SRE_API int LV_SRE_WaitForDecode(HPORT hport, int voicechannel);

/* LV_SRE_DTMFDecode -- Uses active dtmf-mode grammars to parse a DTMF string and return a semantic interpretation in LV_DTMF_CHANNEL
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode
 int flags : unused in the current implementation */
LV_SRE_API int LV_SRE_DTMFDecode(HPORT hport, const char* dtmf_str, unsigned int flags);

/* LV_SRE_InterpretText -- Uses active voice-mode grammars to parse a text string and return a semantic interpretation in LV_INTERPRET_CHANNEL
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode
 int flags : unused in the current implementation */
LV_SRE_API int LV_SRE_InterpretText(HPORT hport, const char* text, unsigned int flags);

/* End SRE request routines 
 **************************************************/

/*****************************************************************
* SRE result handling routines
* 
* Overview:
* After a decode or interpretation has finished, the result is available in the voice channel used.
* A speech recognition result may return several N-best results, each containing a different search hypothesis,
* in descending rank of confidence score.
* Each N-best result may have more than one valid parse through its SRGS grammar, each accessible by index
* Each N-best result may also have more than one semantic interpretation, resulting from applying semantic tags from various parses
*/

/* LV_SRE_GetNumberOfNBestAlternatives : Returns the number of N-best Alternatives generated for the SRE output.
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode*/
LV_SRE_API int LV_SRE_GetNumberOfNBestAlternatives(HPORT hport, int VoiceChannel);

/* LV_SRE_SwitchToNBestAlternative : Used to access alternative SRE parses and interpretations by N-best index
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode
 int Index:	A value from 0 to GetNumberOfNBestAlternatives - 1*/
LV_SRE_API int LV_SRE_SwitchToNBestAlternative(HPORT hport, int VoiceChannel, int Index);


/* LV_SRE_GetNumberOfParses : Returns the number of parses for the SRE output.
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode
 Return Value == 0: the decode didn't match the active grammar(s).
 Return Value > 1: the active grammar(s) were ambiguous for the decode.*/
LV_SRE_API int LV_SRE_GetNumberOfParses(HPORT hport, int VoiceChannel);

/* LV_SRE_GetParseTreeString : Returns a logical parse string for the decode.
 * See "http://www.w3.org/TR/speech-grammar/#AppH"
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode
 int Index:	A value from 0 to GetNumberOfParses - 1, indexing the parse trees for the decode.*/
LV_SRE_API const char* LV_SRE_GetParseTreeString(HPORT hport, int VoiceChannel, int Index);


/* LV_SRE_GetNumberOfInterpretations : Returns the number of semantic intepretations for the SRE output.
HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode*/
LV_SRE_API int LV_SRE_GetNumberOfInterpretations(HPORT hport, int VoiceChannel);


/* LV_SRE_GetInterpretationString : Returns the semantic interpretation result in XML format
 * See "http://www.w3.org/TR/semantic-interpretation/#SI7."
 * Note:  The flag LV_DECODE_SEMANTIC_INTERPRETATION must be set if you want to receive semantic interpretation results.
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode
 int Index:	The index of the interpretation, ranged from 0 through LV_SRE_GetNumberOfInterpretations - 1*/
LV_SRE_API const char* LV_SRE_GetInterpretationString(HPORT hport, int VoiceChannel, int Index);

/* LV_SRE_GetInterpretationScore : Returns the confidence score, from 0-1000 for each interpretation index
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode
 int Index:	The index of the interpretation, ranged from 0 through LV_SRE_GetNumberOfInterpretations - 1*/
LV_SRE_API int LV_SRE_GetInterpretationScore(HPORT hport, int VoiceChannel, int Index);

/* LV_SRE_GetInterpretationGrammarLabel : Gets the label of the grammar used for each interpretation index
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode
 int Index:	The index of the interpretation, ranged from 0 through LV_SRE_GetNumberOfInterpretations - 1*/
LV_SRE_API const char* LV_SRE_GetInterpretationGrammarLabel(HPORT hport, int VoiceChannel, int Index);

/* End SRE result handling routines 
 **************************************************/


/*****************************************************************
* Miscellaneous API routines
* 
* Overview:
  The first three functions are used to get a descriptive error message after a failure is returned
  from a call to the API. Most API calls return an integer that can be used with LV_SRE_ReturnErrorString
  to generate an error string. Because of the more complex nature of Decode and LoadGrammar requests,
  there are additional functions to get error information following those types of requests.
*/

/* LV_SRE_ReturnErrorString: Used to convert a return code for calls to the LV_SRE API into a descriptive message string*
 int ReturnCode: The integer value returned from a call to the LV_SRE API */
LV_SRE_API const char* LV_SRE_ReturnErrorString(int ReturnCode);

/*	LV_SRE_ReturnGrammarErrorString: Used after a LoadGrammar returns LV_GRAMMAR_SYNTAX_ERROR or LV_GRAMMAR_SYNTAX_WARNING to get a description of the error
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 const char* GrammarLabel : the label that was specified in the failed call to LV_SRE_LoadGrammar*/
LV_SRE_API const char* LV_SRE_ReturnGrammarErrorString(HPORT hport, const char* GrammarLabel);

/*	LV_SRE_GetLastDecodeError: For use in case LV_DECODE_BLOCK is not used. Should be called after WaitForEngineToIdle returns success
 HPORT hport : the port handle obtained from a previous call to LV_SRE_CreateClient 
 int VoiceChannel : the VoiceChannel that was specified in the previous call to LV_SRE_Decode*/
LV_SRE_API int LV_SRE_GetLastDecodeError(HPORT hport, int VoiceChannel);

/*	LV_SRE_GetAvailableLicensesCount : Gets the sum of the licenses currently available with all of the connected license servers*/
LV_SRE_API unsigned int LV_SRE_GetAvailableLicensesCount(const char *license_type);

/* End miscellaneous API routines 
 **************************************************/


#if defined(__cplusplus)
}
#endif
//*********************************************************************
#endif
