#ifndef LV_SRE_GRAMMAR_H
#define LV_SRE_GRAMMAR_H

#include <LV_SRE_Defines.h>
#include "LV_SRE_Deprecated.h"
#include "LV_SRE_ParseTree.h"
#include "LV_SRE_Semantic.h"


#ifdef   __cplusplus
extern "C"
{
#endif /*__cplusplus */

/* Function: LVGrammar_Create();
 *
 * Create a grammar object.
 */
LV_SRE_API HGRAMMAR LVGrammar_Create(void);
LV_SRE_API HGRAMMAR LVGrammar_CreateFromCopy(HGRAMMAR Other);
LV_SRE_API void     LVGrammar_Copy(HGRAMMAR Grammar, HGRAMMAR Other);

/* Function: LVGrammar_Release
 *
 * Release a grammar object
 *
 * Input: Grammar	The handle of the object being released.
 */
LV_SRE_API void     LVGrammar_Release(HGRAMMAR Grammar);

/* Function: LVGrammar_Release
 *
 * Clears the contents of a grammar
 *
 * Input: Grammar	The handle of the object being released.
 */
LV_SRE_API int      LVGrammar_Reset(HGRAMMAR Grammar);

/* Function: LVGrammar_LoadGrammar
 *
 * Load the grammar object by compiling an SRGS-ABNF grammar file in a local or remote location.
 * URI schemes supported are file: http: and ftp:
 * Additionaly, you may load using a local posix or dos file locator: (c:\dir\file, ..\dir\file, /tmp/file, etc.)
 *
 * Input: Grammar	The handle of the grammar.
 * Input: uri		The location of the file you are loading
 */
LV_SRE_API int      LVGrammar_LoadGrammar(HGRAMMAR Grammar, const char* uri);

/* Function: LVGrammar_LoadGrammarFromBuffer 
 *
 * Load the grammar from a null terminated string that contains an SRGS-ABNF grammar.
 * Note: if you have external rule references with relative paths in your grammar, you need to include a base uri
 * in your grammar.
 *
 * Input: Grammar	The handle of the grammar.
 * Input: buffer	A null terminated string containing the grammar file.
 * 
 */
LV_SRE_API int      LVGrammar_LoadGrammarFromBuffer(HGRAMMAR Grammar, const char* buffer);

/* Function: LVGrammar_AddRule 
 *
 * Add a rule to your grammar.  
 *
 * Calling LVGrammar_AddRule(grammar, "lhs", "example sentence") is equivalent to
 *		$lhs = example sentence;
 * in an SRGS-ABNF file.
 *
 * Input: Grammar			The handle of the grammar.
 * Input: left_hand_side	The name of the rule ("$" is optional).
 * Input: right_hand_side	The rule expansion.
 */
LV_SRE_API int      LVGrammar_AddRule(HGRAMMAR Grammar, const char* left_hand_side, const char* right_hand_side);

/* Function: LVGrammar_RemoveRule
 * 
 * Remove a rule from your grammar.
 * 
 * Input: Grammar			The handle of the grammar.
 * Input: left_hand_side	The name of the rule being removed ("$" is optional).
 */
LV_SRE_API int      LVGrammar_RemoveRule(HGRAMMAR Grammar, const char* left_hand_side);

/* Function: LVGrammar_SetRoot
 *
 * Set one of your grammar rules as the root rule. 
 * 
 * Calling LVGrammar_SetRoot(grammar, "rule_name") is equivalent to
 *		root $rule_name;
 * in an SRGS-ABNF file.
 *
 * A root rule is the rule the SRE must start in when recognizing an utterance.
 *
 * Note: There is only ever one root rule.  If you call SetRoot multiple times, the most recent call is the root
 * rule.  If you do not call SetRoot, then the grammar will allow all of your rules to act as root rules, by 
 * creating an unnamed rule that is allowed to expand out to any of your grammar's rules.
 *
 * Input: Grammar			The handle of the grammar.
 * Input: root				The name of the rule that will act as the root ("$" is optional).
 */
LV_SRE_API int      LVGrammar_SetRoot(HGRAMMAR Grammar, const char* root);

/* Function: LVGrammar_SetLanguage and LVGrammar_GetLanguage
 *
 * Identifies the language used in this grammar.  
 * The language is supplied as a language or language-COUNTRY pair ("en", "es-MX", etc.)
 *
 * Calling LVGrammar_SetLanguage("en-US") is equivalent to
 *		language en-US;
 * In an SRGS-ABNF grammar.
 *
 * Note:  Currently, the SRE only supports American English, Spanish, and a beta
 *        French Canadian model.  So only the following codes are accepted during decode:
 *
 *        "en", "en-US" ----------> American English
 *        "es", "es-MX" ----------> Spanish
 *        "fr", "fr-CA" ----------> French
 *
 * Input: Grammar			The grammar handle.
 * Input: language			The language or language-COUNTRY code for your grammar.
 * Default: "en-US"
 */
LV_SRE_API int         LVGrammar_SetLanguage(HGRAMMAR Grammar, const char* language);
LV_SRE_API const char* LVGrammar_GetLanguage(HGRAMMAR Grammar);

/* Function: LVGrammar_SetMode and LVGrammar_GetMode
 *
 * Identifies the input mode that the grammar will be recognizing against. 
 * Calling LVGrammar_SetMode("voice") is equivalent to
 *		mode voice;
 * In an SRGS-ABNF grammar.
 *
 * Current legal values are:
 *		"voice"
 *      "dtmf"
 *
 * Input: Grammar	The grammar handle.
 * Input: mode		the input mode of the grammar.
 * Default: "voice"
 */
LV_SRE_API int         LVGrammar_SetMode(HGRAMMAR Grammar, const char* mode);
LV_SRE_API const char* LVGrammar_GetMode(HGRAMMAR Grammar);

/* Function: LVGrammar_SetTagFormat and LVGrammar_GetTagFormat
 *
 * Identifies the format spec of the tags in your grammar.
 *
 * Calling LVGrammar_SetTagFormat("my_tag_format") is equivalent to 
 *		tag-format <my_tag_format>; 
 * in an SRGS-ABNF grammar.
 *
 * To use The LumenVox semantic interpreter, you must specify a tag format compatible with
 * the semantic interpreter.
 * The LumenVox tag format tracks the current W3C Semantic Interpretation working draft,
 * So you may use the following identifiers:
 *
 *		"lumenvox/1.0" or "semantics/1.0"
 *			|-------> Use W3C SI working draft 1 April 2003
 *
 *      "semantics/1.0.2006" or "semantics/1.0.2006-literals"
 *			|-------> Use W3C SI most current draft implementation
 *                    (Currently working draft 3 November 2006)
 *
 * As the W3C SI editors release new versions of the draft, and LumenVox implements them
 * LumenVox will increment its tag version number, and will point "semantics/1.0.xxxx" to the most current 
 * version, allowing us to maintain backward compatibility with previous drafts.
 *
 * Input: Grammar			The grammar handle.
 * Input: tag_format		The tag format identifier.
 * Default: "lumenvox/1.0"
 */
LV_SRE_API int          LVGrammar_SetTagFormat(HGRAMMAR Grammar, const char* tag_format);
LV_SRE_API const char*  LVGrammar_GetTagFormat(HGRAMMAR Grammar);

/* Functions: LVGrammar_ParseSentence, LVGrammar_GetNumberOfParses, LVGrammar_GetParseTree
 * 
 * Grammars can be tested through the api by providing a sentence to be parsed.  The grammar will
 * create a parse tree for each legal parse of the sentence provided.  See LV_SRE_ParseTree.h for 
 * the parse tree API.
 */
LV_SRE_API int          LVGrammar_ParseSentence(HGRAMMAR Grammar, const char* sentence);
LV_SRE_API int          LVGrammar_GetNumberOfParses(HGRAMMAR Grammar);
LV_SRE_API H_PARSE_TREE LVGrammar_CreateParseTree(HGRAMMAR Grammar, int index);

/* If your grammar contains legal Semantic Interpretation tags 
 * (http://www.w3.org/TR/semantic-interpretation/), the parse trees generated by the grammar can be
 * interpreted, to create a LVInterpretation object.
 * See LV_SRE_Semantic.h for the LVInterpretation API
 */
LV_SRE_API int          LVGrammar_InterpretParses(HGRAMMAR Grammar);
LV_SRE_API int          LVGrammar_GetNumberOfInterpretations(HGRAMMAR Grammar);
LV_SRE_API H_SI         LVGrammar_CreateInterpretation(HGRAMMAR Grammar, int index);


LV_SRE_API int			LVGrammar_GetNumberOfMetaData(HGRAMMAR Grammar);
LV_SRE_API const char*	LVGrammar_GetMetaDataKey(HGRAMMAR Grammar, int index);
LV_SRE_API const char*	LVGrammar_GetMetaDataValue(HGRAMMAR Grammar, int index);


LV_SRE_API void		LVGrammar_RegisterLoggingCallback(HGRAMMAR Grammar,GrammarLogCB Log, void* UserData);

#ifdef   __cplusplus
}
#endif /*__cplusplus */




#ifdef __cplusplus

/*	class LVGrammar
 *
 *  C++ inline wrapper of the C api.  
 *  You may bypass this class and call C api directly at any time.
 *
 *  For documentation on the class methods, see the corresponding C method documentation
 */
class LVGrammar
{
public:
	LVGrammar();
	LVGrammar(GrammarLogCB Log, void* UserData);
	LVGrammar& operator=(const LVGrammar& other);
	LVGrammar(const LVGrammar& other);      
	~LVGrammar();

	int LoadGrammar(const char* uri);
	int LoadGrammarFromBuffer(const char* buffer);
	int AddRule(const char* left_hand_side, const char* right_hand_side);
	int RemoveRule(const char* left_hand_side);
	int SetRoot(const char* root);
	int SetLanguage(const char* language);
	const char* GetLanguage() const;
	int SetMode(const char* mode);
	const char* GetMode() const;
	int SetTagFormat(const char* tag_format);
	const char* GetTagFormat() const;
	int GetNumberOfMetaData() const;
	const char* GetMetaDataKey(int index) const;
	const char* GetMetaDataValue(int index) const;

	int Reset();

	void RegisterLoggingCallback(GrammarLogCB Log,void* UserData);

	int ParseSentence(const char* sentence);
	int NumberOfParses();
	LVParseTree GetParseTree(int index);

	int InterpretParses();
	int NumberOfInterpretations();
	LVInterpretation GetInterpretation(int index);

	HGRAMMAR GetHGrammar();

private:
	HGRAMMAR Grammar;
};

//=====================================================================================================================
//
// Inlines
//
//=====================================================================================================================

inline
LVGrammar::LVGrammar() : Grammar(NULL)
{
	Grammar = LVGrammar_Create();
}

//=====================================================================================================================

inline
LVGrammar::LVGrammar(GrammarLogCB Log, void* UserData) : Grammar(NULL)
{
	Grammar = LVGrammar_Create();
	LVGrammar_RegisterLoggingCallback(Grammar,Log,UserData);
}

//=====================================================================================================================

inline
LVGrammar::~LVGrammar()
{
	LVGrammar_Release(Grammar);
}

//=====================================================================================================================

inline
int LVGrammar::Reset()
{
	return LVGrammar_Reset(Grammar);
}

//=====================================================================================================================

inline
int LVGrammar::LoadGrammar(const char* uri)
{
	return LVGrammar_LoadGrammar(Grammar, uri);
}

//=====================================================================================================================

inline
int LVGrammar::LoadGrammarFromBuffer(const char* buffer)
{
	return LVGrammar_LoadGrammarFromBuffer(Grammar, buffer);
}

//=====================================================================================================================

inline
int LVGrammar::AddRule(const char* left_hand_side, const char* right_hand_side)
{
	return LVGrammar_AddRule(Grammar, left_hand_side, right_hand_side);
}

//=====================================================================================================================

inline
int LVGrammar::RemoveRule(const char* left_hand_side)
{
	return LVGrammar_RemoveRule(Grammar, left_hand_side);
}

//=====================================================================================================================

inline
int LVGrammar::SetRoot(const char* root)
{
	return LVGrammar_SetRoot(Grammar, root);
}

//=====================================================================================================================

inline
int LVGrammar::SetLanguage(const char* language)
{
	return LVGrammar_SetLanguage(Grammar, language);
}

//=====================================================================================================================

inline
const char* LVGrammar::GetLanguage() const
{
	return LVGrammar_GetLanguage(Grammar);
}

//=====================================================================================================================

inline
int LVGrammar::SetMode(const char* mode)
{
	return LVGrammar_SetMode(Grammar, mode);
}

//=====================================================================================================================

inline
const char* LVGrammar::GetMode() const
{
	return LVGrammar_GetMode(Grammar);
}

//=====================================================================================================================

inline
int LVGrammar::SetTagFormat(const char* tag_format)
{
	return LVGrammar_SetTagFormat(Grammar, tag_format);
}

//=====================================================================================================================

inline
const char* LVGrammar::GetTagFormat() const
{
	return LVGrammar_GetTagFormat(Grammar);
}

//=====================================================================================================================

inline 
HGRAMMAR LVGrammar::GetHGrammar()
{
	return Grammar;
}

//=====================================================================================================================

inline 
void LVGrammar::RegisterLoggingCallback(GrammarLogCB Log, void* UserData)
{
	LVGrammar_RegisterLoggingCallback(Grammar,Log,UserData);
}

//=====================================================================================================================

inline
int LVGrammar::ParseSentence(const char* sentence)
{
	return LVGrammar_ParseSentence(Grammar,sentence);
}
	
//=====================================================================================================================

inline
int LVGrammar::NumberOfParses()
{
	return LVGrammar_GetNumberOfParses(Grammar);
}

//=====================================================================================================================

inline
LVParseTree LVGrammar::GetParseTree(int index)
{
	return LVParseTree(LVGrammar_CreateParseTree(Grammar,index));
}

//=====================================================================================================================

inline
int LVGrammar::InterpretParses()
{
	return LVGrammar_InterpretParses(Grammar);
}

//=====================================================================================================================

inline
int LVGrammar::NumberOfInterpretations()
{
	return LVGrammar_GetNumberOfInterpretations(Grammar);
}

//=====================================================================================================================

inline
LVInterpretation LVGrammar::GetInterpretation(int index)
{
	return LVInterpretation(LVGrammar_CreateInterpretation(Grammar,index));
}

//=====================================================================================================================

inline
int LVGrammar::GetNumberOfMetaData() const
{
	return LVGrammar_GetNumberOfMetaData(Grammar);
}

//=====================================================================================================================
inline
const char* LVGrammar::GetMetaDataKey(int index) const
{
	return LVGrammar_GetMetaDataKey(Grammar, index);
}

//=====================================================================================================================
inline
const char* LVGrammar::GetMetaDataValue(int index) const
{
	return LVGrammar_GetMetaDataValue(Grammar, index);
}

//=====================================================================================================================

inline
LVGrammar::LVGrammar(const LVGrammar& other):
Grammar(LVGrammar_CreateFromCopy(other.Grammar))
{}

inline
LVGrammar& LVGrammar::operator= (const LVGrammar& other)
{
	if (this != &other)
	{
		LVGrammar_Release(Grammar);
		Grammar = LVGrammar_CreateFromCopy(other.Grammar);
		//LVGrammar_Copy(Grammar,other.Grammar);
	}
	return *this;
}


#endif /*__cplusplus */


#endif /*LV_SRE_GRAMMAR_H */
