#ifndef LV_SRE_SEMANTIC_H
#define LV_SRE_SEMANTIC_H

#include <LV_SRE_Defines.h>
#include "LV_SRE_Deprecated.h"
#include "LV_SRE_ParseTree.h"


/*
 * C interface
 */
#ifdef __cplusplus
extern "C"
{
#endif
LV_SRE_API H_SI        LVInterpretation_Create(void);
LV_SRE_API void        LVInterpretation_Copy(H_SI hsi, H_SI other);
LV_SRE_API H_SI        LVInterpretation_CreateFromCopy(H_SI hsi);
LV_SRE_API H_SI        LVInterpretation_CreateFromParseTree(H_PARSE_TREE Tree, SIErrorCallback callback, void* userdata);
LV_SRE_API void        LVInterpretation_Release(H_SI hsi);
LV_SRE_API const char* LVInterpretation_GetLanguage(H_SI hsi);
LV_SRE_API const char* LVInterpretation_GetMode(H_SI hsi);
LV_SRE_API const char* LVInterpretation_GetTagFormat(H_SI hsi);
LV_SRE_API const char* LVInterpretation_GetGrammarLabel(H_SI hsi);
LV_SRE_API int         LVInterpretation_GetScore(H_SI hsi);
LV_SRE_API const char* LVInterpretation_GetResultName(H_SI hsi);
LV_SRE_API const char* LVInterpretation_GetInputSentence(H_SI hsi);
LV_SRE_API H_SI_DATA   LVInterpretation_GetResultData(H_SI hsi);

/* 
 * Clone the semantic data from another handle. 
 * This is function allocates memory for the new handle,
 * So make sure to release it.
 */
LV_SRE_API	H_SI_DATA	LVSemanticData_CreateFromCopy(H_SI_DATA h_si_data);

/* DEPRECATED Function -- Use LVInterpretation_CreateFromParseTree
 * Create a semantic data handle from a parse tree handle.
 * SIErrorCallback is the call back function for error messages.
 * This is function allocates memory for the new handle,
 * So make sure to release it.
 */
LV_SRE_API	H_SI_DATA	SI_DATA_Create_From_HSPT(H_PARSE_TREE tree, SIErrorCallback callback, void *userdata);

/*
 * Release a handle
 */
LV_SRE_API	void LVSemanticData_Release(H_SI_DATA h_si_data);

/*
 * Return the type of the data
 */
LV_SRE_API	int LVSemanticData_GetType(H_SI_DATA h_si_data);

/*
 * Print the content of the handle in specified format
 */
LV_SRE_API	const char* LVSemanticData_Print(H_SI_DATA h_si_data, int format);

/*
 * Return the value of SI_TYPE_BOOL type semantic data.
 * For other types, this function always returns false.
 * It is recommended to verify the type before use this function.
 */
LV_SRE_API	int LVSemanticData_GetBool(H_SI_DATA h_si_data);

/*
 * Return the value of SI_TYPE_INT type semantic data.
 * For other types, this function always returns 0.
 * It is recommended to verify the type before use this function.
 */
LV_SRE_API	int	LVSemanticData_GetInt(H_SI_DATA h_si_data);

/*
 * Return the value of SI_TYPE_DOUBLE type semantic data.
 * For other types, this function always returns 0.0.
 * It is recommended to verify the type before use this function.
 */
LV_SRE_API	double LVSemanticData_GetDouble(H_SI_DATA h_si_data);

/*
 * Return the value of SI_TYPE_STRING type semantic data.
 * For other types, this function always returns NULL.
 * It is recommended to verify the type before use this function.
 */
LV_SRE_API	const char* LVSemanticData_GetString(H_SI_DATA h_si_data);

/*
 * Return the number of properties of SI_TYPE_OBJECT type semantic data.
 * For other types, this function always returns -1.
 * It is recommended to verify the type before use this function.
 */
LV_SRE_API	int LVSemanticObject_GetNumberOfProperties(H_SI_DATA h_si_data);

/*
 * Return property name of SI_TYPE_OBJECT type semantic data.
 * If the index is greater or equal to SI_DATA_Object_Size()-1,
 * this function returns NULL.
 * For other types, this function always returns NULL.
 * It is recommended to verify the type before use this function.
 */
LV_SRE_API	const char* LVSemanticObject_GetPropertyName(H_SI_DATA h_si_data, int index);

/*
 * Return the semantic data handle associated with a property name 
 * in an SI_TYPE_OBJECT type semantic data.
 * If the property name doesn't exist, this function returns NULL.
 * For other types, this function always returns NULL.
 * It is recommended to verify the type before use this function.
 * NOTE: this function doesn't allocate memory, so do not release 
 * the new handle.
 */
LV_SRE_API	H_SI_DATA	LVSemanticObject_GetPropertyValue(H_SI_DATA h_si_data, const char* property_id);

/*
 * Check if the property name exists in an SI_TYPE_OBJECT type semantic data.
 * For other types, this function always returns false.
 * It is recommended to verify the type before use this function.
 */
LV_SRE_API	int LVSemanticObject_PropertyExists(H_SI_DATA h_si_data, const char* property_id);

/*
 * Return the number of elements of SI_TYPE_ARRAY type semantic data.
 * For other types, this function always returns -1.
 * It is recommended to verify the type before use this function.
 */
LV_SRE_API	int LVSemanticArray_GetSize(H_SI_DATA h_si_data);

/*
 * Return the semantic data handle indicated by the index 
 * in an SI_TYPE_ARRAY type semantic data.
 * If the index is greater or equal to SI_DATA_Array_Size()-1, 
 * this function returns NULL.
 * For other types, this function always returns NULL.
 * It is recommended to verify the type before use this function.
 * NOTE: this function doesn't allocate memory, so do not release 
 * the new handle.
 */
LV_SRE_API	H_SI_DATA	LVSemanticArray_GetElement(H_SI_DATA h_si_data, int index);

/*
 * Check if the two objects indicated by the two handles are equal to each other
 */
LV_SRE_API	int LVSemanticData_AreEqual(H_SI_DATA h_si_data1, H_SI_DATA h_si_data2);

#ifdef __cplusplus
}
#endif

/*
 * C++ interface
 */
#ifdef __cplusplus

class LVSemanticObject;
class LVSemanticArray;
class LVSemanticData;


/*
 * The class presenting semantic data
 */
class LVSemanticData
{
public:
	LVSemanticData();

	//deprecated.  Use LVInterpretation constructor.
	LVSemanticData(const LVParseTree & tree, SIErrorCallback callback = NULL, void *userdata = NULL);

	LVSemanticData(const LVSemanticData & other);

	LVSemanticData(H_SI_DATA hsi);

	LVSemanticData & operator = (const LVSemanticData & other);
	
	~LVSemanticData();

	/*
	 * return a H_SI_DATA handle
	 * NOTE: this method doesn't allocate memory, so do not release 
	 * the new handle.
	 */
	H_SI_DATA Get_H_SI_DATA() const {return Handle();}//deprecated name

	H_SI_DATA Handle() const;

	/*
	 * return the type.
	 */
	int Type() const;

	/*
	 * return bool value if the type is SI_TYPE_BOOL.
	 * If the type is not SI_TYPE_BOOL, this method always returns false
	 */
	bool GetBool() const;

	/*
	 * return integer value if the type is SI_TYPE_INT.
	 * If the type is not SI_TYPE_INT, this method always returns 0
	 */
	int GetInt() const;

	/*
	 * return double value if the type is SI_TYPE_DOUBLE.
	 * If the type is not SI_TYPE_DOUBLE, this method always returns 0.0
	 */
	double GetDouble() const;

	/*
	 * return string value if the type is SI_TYPE_STRING.
	 * If the type is not SI_TYPE_STRING, this method always returns NULL
	 */
	const char* GetString() const;

	/*
	 * return a LVSemanticObject object if the type is SI_TYPE_OBJECT.
	 * If the type is not SI_TYPE_OBJECT, this method will return
	 * an object with size equal to zero.
	 */
	const LVSemanticObject & GetSemanticObject() const;
	const LVSemanticObject & GetObject() const;//deprecated name

	/*
	 * return a LVSemanticArray object if the type is SI_TYPE_ARRAY.
	 * If the type is not SI_TYPE_ARRAY, this method will return
	 * an array with size equal to zero.
	 */
	const LVSemanticArray & GetSemanticArray() const;
	const LVSemanticArray & GetArray() const;//deprecated name

	/*
	 * Print the content of the data in specified format
	 */
	const char* Print(int format) const;
private:
	H_SI_DATA	data;
	
	mutable LVSemanticObject *pobject;
	mutable LVSemanticArray *parray;


friend class LVSpeechPort;
friend class LVSemanticObject;
friend class LVSemanticArray;
friend class LVInterpretation;
};

bool operator == (const LVSemanticData& lhs, const LVSemanticData& rhs);

inline
bool operator != (const LVSemanticData& lhs, const LVSemanticData& rhs) {return !(lhs == rhs);}


/*
 * The class presenting SI_TYPE_OBJECT type semantic data
 */
class LVSemanticObject
{
public:
	LVSemanticObject();

	LVSemanticObject(const LVSemanticObject & other);

	LVSemanticObject & operator = (const LVSemanticObject & other);

	~LVSemanticObject();

	/*
	 * return the number of propeties
	 */
	int Size() const {return NumberOfProperties();}//deprecated name
	int NumberOfProperties() const;

	/*
	 * return the property name
	 */
	const char* PropertyId(int Index) const {return PropertyName(Index);}//deprecated name
	const char* PropertyName(int Index) const;

	/*
	 * return a LVSemanticData object associated with the property name
	 * If the property name doesn't exist, the return object will has type
	 * LV_TYPE_NULL.
	 */
	const LVSemanticData & operator [] (const char* PropertyId) const;

	/*
	 * return a LVSemanticData object associated with the property name
	 * If the property name doesn't exist, the return object will has type
	 * LV_TYPE_NULL.
	 */
	const LVSemanticData & PropertyValue(const char* PropertyId) const;
	const LVSemanticData & PropertyValue(int Index) const;

	/*
	 * Check if a propety name exists in this object.
	 */
	bool PropertyExists(const char* PropertyId) const;

	/*
	 * Print the content of the object in specified format
	 */
	const char* Print(int format) const;

	H_SI_DATA Handle() const{return data;}
private:
	H_SI_DATA	data;
	mutable LVSemanticData semantic_data;

friend class LVSemanticData;
friend bool operator == (const LVSemanticObject & lhs, const LVSemanticObject & rhs);
friend bool operator != (const LVSemanticObject & lhs, const LVSemanticObject & rhs);
};

bool operator == (const LVSemanticObject & lhs, const LVSemanticObject & rhs);

inline
bool operator != (const LVSemanticObject & lhs, const LVSemanticObject & rhs) {return !(lhs == rhs);}

/* 
 * The class presenting SI_TYPE_ARRAY type semantic data
 */
class LVSemanticArray
{
public:
	LVSemanticArray();

	LVSemanticArray(const LVSemanticArray &other);

	LVSemanticArray & operator = (const LVSemanticArray & other);

	~LVSemanticArray();

	/*
	 * return number of elements
	 */
	int Size() const;

	/*
	 * return a LVSemanticData object indexed by paramenter 'Index'
	 * If 'Index' is greater or equal to Size()-1, this method returns
	 * an object with type LV_TYPE_NULL.
	 */
	const LVSemanticData & operator [] (int Index) const;

	/*
	 * return a LVSemanticData object indexed by paramenter 'Index'
	 * If 'Index' is greater or equal to Size()-1, this method returns
	 * an object with type LV_TYPE_NULL.
	 */
	const LVSemanticData & At(int Index) const;

	/*
	 * Print the content of the array in specified format
	 */
	const char* Print(int format) const;

	H_SI_DATA Handle() const{return data;}

private:
	H_SI_DATA data;
	mutable LVSemanticData semantic_data;

friend class LVSemanticData;
friend bool operator == (const LVSemanticArray & lhs, const LVSemanticArray & rhs);
friend bool operator != (const LVSemanticArray & lhs, const LVSemanticArray & rhs);
};

/*
 * Represents the results of the recognition and interpretation process.
 * contains:
 *     InputSentence: The spoken input (or DTMF string) the engine recognized.
 *     GrammarLabel:  The name of the grammar that the input matched.
 *     Score:         The utterance confidence score
 *     Mode:          The interaction mode (was this a dtmf or vocal transaction)
 *     Language:      The language of grammar and acoustic models.
 *     TagFormat:     The name and version of the semantic procedure that processed the input.
 *     Result:        The output of the semantic process: A SemanticData object
 *     ResutlName:    The name of the result object (usually the root rule name of the matching grammar).
 */
class LVInterpretation
{
public:
	LVInterpretation();
	LVInterpretation(const LVParseTree& Tree, SIErrorCallback callback = NULL, void* userdata = NULL);
	LVInterpretation(const LVInterpretation& Other);

	//constructor takes control of handle.  Do not release
	LVInterpretation(H_SI handle);
	~LVInterpretation();

	LVInterpretation& operator=(const LVInterpretation& Other);

	const char* GrammarLabel() const;
	const char* Language() const;
	const char* Mode() const;
	const char* TagFormat() const;
	const char* InputSentence() const;
	int Score() const;

	const char* ResultName() const;
	const LVSemanticData& ResultData() const;

	H_SI Handle() const;

private:
	LVSemanticData r;
	H_SI si;
};

bool operator == (const LVSemanticArray & lhs, const LVSemanticArray & rhs);

inline
bool operator != (const LVSemanticArray & lhs, const LVSemanticArray & rhs){return !(lhs == rhs);}

//======================================================================================================================
inline
LVSemanticData::LVSemanticData():
data(NULL),
pobject(NULL),
parray(NULL)
{

}

//======================================================================================================================
inline
LVSemanticData::LVSemanticData(const LVParseTree &tree, SIErrorCallback callback, void *userdata):
pobject(NULL),
parray(NULL)
{
	data = SI_DATA_Create_From_HSPT(tree.Handle(), callback, userdata);

	if (!pobject) pobject = new LVSemanticObject();
	if (!parray) parray = new LVSemanticArray();

	if (LVSemanticData_GetType(data) == SI_TYPE_OBJECT) pobject->data = data;
	if (LVSemanticData_GetType(data) == SI_TYPE_ARRAY) parray->data = data;
}
//======================================================================================================================
inline
LVSemanticData::LVSemanticData(H_SI_DATA hsi):
data(LVSemanticData_CreateFromCopy(hsi)),
pobject(NULL),
parray(NULL)
{
	if (!pobject) pobject = new LVSemanticObject();
	if (!parray) parray = new LVSemanticArray();

	if (LVSemanticData_GetType(data) == SI_TYPE_OBJECT) pobject->data = data;
	if (LVSemanticData_GetType(data) == SI_TYPE_ARRAY) parray->data = data;
}

//======================================================================================================================
inline
LVSemanticData::LVSemanticData(const LVSemanticData &other):
data(NULL),
pobject(NULL),
parray(NULL)
{
	data = LVSemanticData_CreateFromCopy(other.Handle());

	if (!pobject) pobject = new LVSemanticObject();
	if (!parray) parray = new LVSemanticArray();

	if (LVSemanticData_GetType(data) == SI_TYPE_OBJECT) pobject->data = data;
	if (LVSemanticData_GetType(data) == SI_TYPE_ARRAY) parray->data = data;
}

//======================================================================================================================
inline
LVSemanticData & LVSemanticData::operator = (const LVSemanticData &other)
{
	if (this == &other) return *this;

	if (data)
		LVSemanticData_Release(data);

	data = LVSemanticData_CreateFromCopy(other.Handle());

	if (!pobject) pobject = new LVSemanticObject();
	if (!parray) parray = new LVSemanticArray();

	if (LVSemanticData_GetType(data) == SI_TYPE_OBJECT) 
	{
		pobject->data = data;
		parray->data = NULL;
	}
	if (LVSemanticData_GetType(data) == SI_TYPE_ARRAY)
	{
		parray->data = data;
		pobject->data = NULL;
	}

	return *this;
}

//======================================================================================================================
inline
LVSemanticData::~LVSemanticData()
{
	if (data)
		LVSemanticData_Release(data);

	if (pobject)
	{
		pobject->data = NULL;
		delete pobject;
	}

	if (parray)
	{
		parray->data = NULL;
		delete parray;
	}
}

//======================================================================================================================
inline
H_SI_DATA LVSemanticData::Handle() const
{
	return data;
}

//======================================================================================================================
inline
int LVSemanticData::Type() const
{
	return LVSemanticData_GetType(data);
}

//======================================================================================================================
inline
bool LVSemanticData::GetBool() const
{
	return LVSemanticData_GetBool(data) != 0 ? true : false;
}

//======================================================================================================================
inline
int LVSemanticData::GetInt() const
{
	return LVSemanticData_GetInt(data);
}

//======================================================================================================================
inline
double LVSemanticData::GetDouble() const
{
	return LVSemanticData_GetDouble(data);
}

//======================================================================================================================
inline
const char* LVSemanticData::GetString() const
{
	return LVSemanticData_GetString(data);
}

//======================================================================================================================
inline
const LVSemanticObject & LVSemanticData::GetSemanticObject() const
{

	if (!pobject) pobject = new LVSemanticObject();
	if (Type() == SI_TYPE_OBJECT)
		pobject->data = data;

	return *pobject;
}

inline
const LVSemanticObject & LVSemanticData::GetObject() const
{
	return GetSemanticObject();
}

//======================================================================================================================
inline
const LVSemanticArray & LVSemanticData::GetSemanticArray() const
{
    if (!parray) parray = new LVSemanticArray();
	if (Type() == SI_TYPE_ARRAY)
		parray->data = data;

	return *parray;
}

inline
const LVSemanticArray & LVSemanticData::GetArray() const
{
	return GetSemanticArray();
}

//======================================================================================================================
inline
const char* LVSemanticData::Print(int format) const
{
	return LVSemanticData_Print(data, format);
}

//======================================================================================================================
inline
LVSemanticObject::LVSemanticObject():data(NULL)
{
}

//======================================================================================================================
inline
LVSemanticObject::LVSemanticObject(const LVSemanticObject &other):data(NULL)
{
	data = LVSemanticData_CreateFromCopy(other.data);
}

//======================================================================================================================
inline
LVSemanticObject& LVSemanticObject::operator = (const LVSemanticObject &other)
{
	if (this == &other) return *this;

	if (data)
		LVSemanticData_Release(data);

	data = LVSemanticData_CreateFromCopy (other.data);

	return *this;
}

//======================================================================================================================
inline
LVSemanticObject::~LVSemanticObject()
{
	if (data)
		LVSemanticData_Release(data);


	semantic_data.data = NULL;
}

//======================================================================================================================
inline
int LVSemanticObject::NumberOfProperties() const
{
	return LVSemanticObject_GetNumberOfProperties(data);
}

//======================================================================================================================
inline
bool LVSemanticObject::PropertyExists(const char *PropertyId) const
{
	return LVSemanticObject_PropertyExists(data, PropertyId) != 0 ? true : false;
}

//======================================================================================================================
inline
const char* LVSemanticObject::PropertyName(int Index) const
{
	return LVSemanticObject_GetPropertyName(data, Index);
}

//======================================================================================================================
inline
const LVSemanticData & LVSemanticObject::PropertyValue(const char *PropertyId) const
{
	//LVSemanticData ret;
	//ret.data = LVSemanticData_CreateFromCopy(LVSemanticObject_GetPropertyValue(data, PropertyId));

	//return ret;

	semantic_data.data = LVSemanticObject_GetPropertyValue(data, PropertyId);

	return semantic_data;
}

//======================================================================================================================
inline
const LVSemanticData & LVSemanticObject::PropertyValue(int index) const
{
	return PropertyValue(PropertyName(index));
}

//======================================================================================================================
inline
const LVSemanticData & LVSemanticObject::operator [] (const char *PropertyId) const
{
	return PropertyValue(PropertyId);
}

//======================================================================================================================
inline
const char* LVSemanticObject::Print(int format) const
{
	return LVSemanticData_Print(data, format);
}

//======================================================================================================================
inline
LVSemanticArray::LVSemanticArray():data(NULL)
{
}

//======================================================================================================================
inline
LVSemanticArray::LVSemanticArray(const LVSemanticArray & other):data(NULL)
{
	data = LVSemanticData_CreateFromCopy(other.data);
}

//======================================================================================================================
inline
LVSemanticArray& LVSemanticArray::operator = (const LVSemanticArray & other)
{
	if (this == &other) return *this;

	if (data)
		LVSemanticData_Release(data);

	data = LVSemanticData_CreateFromCopy (other.data);

	return *this;
}

//======================================================================================================================
inline
LVSemanticArray::~LVSemanticArray()
{
	if (data)
		LVSemanticData_Release(data);
	data = NULL;

	semantic_data.data = NULL;
}

//======================================================================================================================
inline
int LVSemanticArray::Size() const 
{
	return LVSemanticArray_GetSize(data);
}

//======================================================================================================================
inline
const LVSemanticData & LVSemanticArray::At(int Index) const
{
	//LVSemanticData ret;
	//ret.data = LVSemanticData_CreateFromCopy(LVSemanticArray_GetElement(data, Index));

	//return ret;

	semantic_data.data = LVSemanticArray_GetElement(data, Index);
	return semantic_data;
}

//======================================================================================================================
inline
const LVSemanticData & LVSemanticArray::operator [] (int Index) const
{
	return At(Index);
}


//======================================================================================================================
inline
const char* LVSemanticArray::Print(int format) const
{
	return LVSemanticData_Print(data, format);
}

//======================================================================================================================
inline
bool operator == (const LVSemanticData & lhs, const LVSemanticData & rhs)
{
	return LVSemanticData_AreEqual(lhs.Handle(), rhs.Handle()) != 0 ? true : false;
}

//======================================================================================================================
inline
bool operator == (const LVSemanticObject & lhs, const LVSemanticObject & rhs)
{
	return LVSemanticData_AreEqual(lhs.data, rhs.data) != 0 ? true : false;
}
//======================================================================================================================
inline
bool operator == (const LVSemanticArray & lhs, const LVSemanticArray & rhs)
{
	return LVSemanticData_AreEqual(lhs.data, rhs.data) != 0 ? true : false;
}

//======================================================================================================================
inline
LVInterpretation::LVInterpretation() : si(NULL),r(NULL)
{}

//======================================================================================================================
inline
LVInterpretation::LVInterpretation(const LVInterpretation& other) :
si(LVInterpretation_CreateFromCopy(other.Handle())), r(other.r)
{
	//si = LVInterpretation_CreateFromCopy(other.Handle());
	//r = other.r;
}

//======================================================================================================================
inline
LVInterpretation::LVInterpretation(const LVParseTree& Tree, 
								   SIErrorCallback callback, 
								   void* userdata):
si(LVInterpretation_CreateFromParseTree(Tree.Handle(),callback,userdata)),
r(NULL)
{
	//si = LVInterpretation_CreateFromParseTree(Tree.Handle(),callback,userdata);
	H_SI_DATA data = LVInterpretation_GetResultData(si);
	if(data)
	{
		r = LVSemanticData(data);
		LVSemanticData_Release(data);
	}
}

//======================================================================================================================
//note: class takes responsibility for handle: do not release!
inline
LVInterpretation::LVInterpretation(H_SI handle):
si(handle),
r(NULL)
{
	//si = handle;
	H_SI_DATA data = LVInterpretation_GetResultData(si);
	if(data)
	{
		r = LVSemanticData(data);
		LVSemanticData_Release(data);
	}

}

//======================================================================================================================
inline
LVInterpretation::~LVInterpretation()
{
	//r.data = NULL;
	LVInterpretation_Release(si);
}

//======================================================================================================================
inline
LVInterpretation& LVInterpretation::operator=(const LVInterpretation& other)
{
	if (si != other.si)
	{
		//r.data = NULL;
		if(si) LVInterpretation_Release(si);
		si = LVInterpretation_CreateFromCopy(other.si);
		//r.data = LVInterpretation_GetResultData(si);
		r = other.r;
	}

	return *this;
}

//======================================================================================================================
inline
const char* LVInterpretation::GrammarLabel() const
{return LVInterpretation_GetGrammarLabel(si);}

//======================================================================================================================
inline
const char* LVInterpretation::Language() const
{return LVInterpretation_GetLanguage(si);}

//======================================================================================================================
inline
const char* LVInterpretation::Mode() const
{return LVInterpretation_GetMode(si);}

//======================================================================================================================
inline
const char* LVInterpretation::TagFormat() const
{return LVInterpretation_GetTagFormat(si);}

//======================================================================================================================
inline
const char* LVInterpretation::InputSentence() const
{return LVInterpretation_GetInputSentence(si);}

//======================================================================================================================
inline
int LVInterpretation::Score() const
{return LVInterpretation_GetScore(si);}

//======================================================================================================================
inline
const char* LVInterpretation::ResultName() const
{return LVInterpretation_GetResultName(si);}

//======================================================================================================================
inline
const LVSemanticData& LVInterpretation::ResultData() const
{return r;}

//======================================================================================================================
inline
H_SI LVInterpretation::Handle() const
{return si;}



#endif//__cplusplus

#endif//LV_SRE_SEMANTIC_H
