#ifndef __LV_TTS_H_2F731CF5_F3C6_4607_9AB6_9B1881D899DC__INCLUDED__
#define __LV_TTS_H_2F731CF5_F3C6_4607_9AB6_9B1881D899DC__INCLUDED__


#include "LV_SRE_Defines.h"



// ===============================================================================================
// The LV_TTS_ interface provides "C" API access to the TTS generation functions exposed by the 
// LumenVox Speech Port.
// 
// The LVTTSClient class provides a lightweight C++ wrapper around this set of API functions as
// described in LVTTSClient.h
// ===============================================================================================



//prevent name-mangling
#if defined(__cplusplus)
extern "C"
{
#endif



// -----------------------------------------------------------------------------------------------
// LV_TTS_CreateClient
//     Create a TTS client object. Necessary to perform any TTS operations.
//
// Inputs:
//   lang_code        : The language of the voice that is to be used for synthesis.
//   gender           : The default gender of the voice that is to be used for synthesis. If
//                      'speaker_name' is not NULL, the gender of the chosen speaker will override
//                      this setting. Possible values are "Male" and "Female".
//   speaker_name     : Specify the name of the speaker whose voice is to be used for synthesis.
//                      If NULL, a general voice for the specified 'gender' will be used.
//   sampling_rate    : The sampling rate (in Hz) the synthesized audio is expected to be
//                      produced in.
//   error_return_code: A variable that would contain a code representing the reason for failure.
//                      Passing in NULL (if you do not care to know the reason for failure) does
//                      not affect the functioning.
//
//   The above inputs only specify the defaults. If the input for synthesis is an SSML document,
//   any additionally required voices are loaded automatically (if available).
//
// Returns:
//   On success - A handle to the TTS client object
//   On failure - NULL
//
HTTSCLIENT LV_TTS_CreateClient(const char * lang_code, const char * gender, const char * speaker_name,
							   unsigned int sampling_rate, LV_TTS_RETURN_CODE * error_return_code);


// -----------------------------------------------------------------------------------------------
// LV_TTS_DestroyClient
//     Release a TTS client object that was created using LV_TTS_CreateClient. Proper destruction
//     of the TTS client object allows for recycling of any license(s) acquired by it.
//
// Inputs:
//   client_handle    : Handle to the TTS client object that is to be destroyed
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_TTS_HANDLE (The input TTS client handle is not a valid one)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_DestroyClient(HTTSCLIENT client_handle);


// -----------------------------------------------------------------------------------------------
// LV_TTS_Synthesize:
//     Perform the actual text to speech operation for the given sentence or SSML text.
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   input_sentence   : Can be a simple sequence of words/sentences or an SSML document
//   flags            : Used to control certain facets of the operation. Mostly unused in the current
//                      implementation besides LV_TTS_BLOCK. Setting the LV_TTS_BLOCK bit in flags causes
//                      LV_TTS_Synthesize to wait for a response back from the TTS server before returning.
//
// Possible return values:
//   On success - LV_SUCCESS
//   On failure - LV_NO_TTS_SERVERS, LV_INVALID_TTS_HANDLE, LV_INVALID_LICENSE_MAINT_DATE,
//                LV_NO_COMPATIBLE_TTS_SERVERS, LV_EXPIRED_LICENSE, LV_RESOURCE_ERROR,
//                LV_FAILED_VOICE_LOAD, LV_FAILED_STREAMER, LV_FAILED_SYNTHESIS,
//                LV_FAILED_AUDIO_CONVERSION, LV_SSML_ERROR, LV_ACQUIRING_LICENSE_FAILED,
//                LV_FAILED_NO_VOICE_SPECIFIED, LV_INVALID_CLIENT_OBJECT, LV_TIMEOUT, LV_EXCEPTION
//
LV_TTS_RETURN_CODE LV_TTS_Synthesize(HTTSCLIENT client_handle, const char * input_sentence, unsigned int flags);


// -----------------------------------------------------------------------------------------------
// LV_TTS_WaitForSynthesis
//     Suspend the current thread until either a response for a previous non-blocking synthesis
//     operation is available or the specified timeout period elapses. 
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   timeout_ms       : Duration (in milliseconds) to wait for the TTS server's response.
//
// Returns:
//   On success - LV_SUCCESS
//   On timeout - LV_TIME_OUT
//   On failure - LV_INVALID_TTS_HANDLE (happens only if the input is invalid) or one of
//                LV_RESOURCE_ERROR, LV_FAILED_VOICE_LOAD, LV_FAILED_STREAMER, LV_FAILED_SYNTHESIS,
//                LV_FAILED_NO_VOICE_SPECIFIED, LV_FAILED_AUDIO_CONVERSION or LV_SSML_ERROR as
//                would be returned by LV_TTS_Synthesize.
//
LV_TTS_RETURN_CODE LV_TTS_WaitForSynthesis(HTTSCLIENT client_handle, unsigned int timeout_ms);


// -----------------------------------------------------------------------------------------------
// LV_TTS_GetSynthesizedAudioBufferLength
//     Get the size of the synthesized audio buffer that was obtained from a LV_TTS_Synthesize call.
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   buffer_length    : Location of memory where the size of the synthesized audio buffer will be copied
//                      into.  This will be a positive integer if successful, or 0 if no results are
//                      available, or the specified client handle is invalid.
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_FAILURE (if the client handle is invalid)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetSynthesizedAudioBufferLength(HTTSCLIENT client_handle, int * buffer_length);


// -----------------------------------------------------------------------------------------------
// LV_TTS_GetSynthesizedAudioBuffer
//     Read parts of the synthesized audio buffer that was obtained from a LV_TTS_Synthesize call. Each successive
//     invocation of the function reads the buffer at an offset that is after the last sample read in the
//     previous call.
//
// Inputs:
//   client_handle        : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   buffer               : Location of memory where the synthesized audio samples will be copied into
//   buffer_size_in_bytes : Amount of memory allocated for 'buffer', number of bytes of synthesized audio
//                          samples to be read into 'buffer'.
//   bytes_returned       : Location of memory where the number of bytes copied into buffer will be copied into.
//                          This may be less than the buffer_size_in_bytes if insufficient data is available.
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_FAILURE (if the client handle is invalid or 'buffer' points to an invalid memory location)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetSynthesizedAudioBuffer(HTTSCLIENT client_handle, void * buffer, int buffer_size_in_bytes, int * bytes_returned);


// -----------------------------------------------------------------------------------------------
// LV_TTS_RewindBuffer
//     Reset the buffer read offset so that the next invocation of LV_TTS_GetSynthesizedAudioBuffer starts
//     from the first sample in the buffer.
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_TTS_HANDLE (happens only if the input is invalid)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_RewindBuffer(HTTSCLIENT client_handle);


// -----------------------------------------------------------------------------------------------
// LV_TTS_GetSynthesizedAudioFormat
//     Query the sound format of the synthesized audio.
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   audio_format     : Memory location into which the audio format is copied. The audio format will be
//                      an integer equivalent to one of the following types: SFMT_ULAW, SFMT_PCM or SFMT_ALAW
//                      or SFMT_UNKNOWN if the sound format is not known or no type information is available.
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_FAILURE (if the client handle is invalid)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetSynthesizedAudioFormat(HTTSCLIENT client_handle, int * audio_format);


// -----------------------------------------------------------------------------------------------
// LV_TTS_GetSynthesizedAudioSampleRate
//     Query the sampling rate of the synthesized audio.
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   sample_rate      : Memory location into which the sample_rate is copied.
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_FAILURE (if the client handle is invalid)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetSynthesizedAudioSampleRate(HTTSCLIENT client_handle, int * sample_rate);


// -----------------------------------------------------------------------------------------------
// LV_TTS_GetSentenceMarksCount
// LV_TTS_GetWordMarksCount
// LV_TTS_GetSSMLMarksCount
//     Query the corresponding number of speech-to-text correlation marks available in the response.
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   return_count     : Memory location into which the number of specified marks is copied. This may be
//                      zero if no information is available from the synthesize request yet.
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_FAILURE (if the client handle is invalid)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetSentenceMarksCount(HTTSCLIENT client_handle, int *return_count);
LV_TTS_RETURN_CODE LV_TTS_GetWordMarksCount(HTTSCLIENT client_handle, int *return_count);
LV_TTS_RETURN_CODE LV_TTS_GetSSMLMarksCount(HTTSCLIENT client_handle, int *return_count);


// -----------------------------------------------------------------------------------------------
// LV_TTS_JumpToSentence
// LV_TTS_JumpToWord
// LV_TTS_JumpToSSMLMark
//     Move the buffer offset to the beginning of the specified sentence/word/ssml_mark so that the next 
//     call to LV_TTS_GetSynthesizedAudioBuffer returns samples starting at that offset.
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   sentence_idx     : Index of the sentence of whose beginning the offset must be moved to.
//   word_idx         : Index of the word of whose beginning the offset must be moved to.
//   ssml_mark_idx    : Index of the SSML mark of whose beginning the offset must be moved to.
//
//   The index values passed to these functions must within the range
//   [ 0 <= idx < the value returned by the corresponding LV_TTS_Get*MarksCount call on success ].
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_TTS_HANDLE (if the client handle is invalid)
//                LV_NO_RESULT_AVAILABLE (if no results are available yet)
//                LV_FAILURE (if the index is out of range)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_JumpToSentence(HTTSCLIENT client_handle, unsigned int sentence_idx);
LV_TTS_RETURN_CODE LV_TTS_JumpToWord(HTTSCLIENT client_handle, unsigned int word_idx);
LV_TTS_RETURN_CODE LV_TTS_JumpToSSMLMark(HTTSCLIENT client_handle, unsigned int ssml_mark_idx);


// -----------------------------------------------------------------------------------------------
// LV_TTS_GetSentenceOffsetInBuffer
// LV_TTS_GetWordOffsetInBuffer
// LV_TTS_GetSSMLMarkOffsetInBuffer
//     Query the offset of the beginning of the specified sentence/word/ssml_mark in the synthesized audio buffer. 
//     Provides the offset relative to the start of the buffer.
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   sentence_idx     : Index of the sentence of whose beginning the offset must be moved to.
//   word_idx         : Index of the word of whose beginning the offset must be moved to.
//   ssml_mark_idx    : Index of the SSML mark of whose beginning the offset must be moved to.
//   buffer_offset    : Location of a variable in which the result will be stored.
//
//   The index values passed to these functions must within the range
//   [ 0 <= idx < the value returned by the corresponding LV_TTS_Get*MarksCount call on success ].
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_TTS_HANDLE (if the client handle is invalid)
//                LV_NO_RESULT_AVAILABLE (if no results are available yet)
//                LV_FAILURE (if the index is out of range)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetSentenceOffsetInBuffer(HTTSCLIENT client_handle, unsigned int sentence_idx, int * buffer_offset);
LV_TTS_RETURN_CODE LV_TTS_GetWordOffsetInBuffer(HTTSCLIENT client_handle, unsigned int word_idx, int * buffer_offset);
LV_TTS_RETURN_CODE LV_TTS_GetSSMLMarkOffsetInBuffer(HTTSCLIENT client_handle, unsigned int ssml_mark_idx, int * buffer_offset);


// -----------------------------------------------------------------------------------------------
// LV_TTS_GetSSMLMarkName
//     Query the name of the specified SSML speech marker, limiting result to name_buffer_length bytes
//
// Inputs:
//   client_handle      : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   ssml_mark_idx      : Index of the SSML mark being specified
//   name_buffer        : Character string array that will be populated with the queried SSML mark name
//   name_buffer_length : Length in bytes of the maximum string length permitted in name_buffer
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_TTS_HANDLE (if the client handle is invalid)
//                LV_NO_RESULT_AVAILABLE (if no results are available yet)
//                LV_FAILURE (if the index is out of range)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetSSMLMarkName(HTTSCLIENT client_handle, unsigned int ssml_mark_idx, char * name_buffer, int name_buffer_length);


// -----------------------------------------------------------------------------------------------
// LV_TTS_IsServerAvailable
//     Check if there are any connected TTS servers are available.
//
// Returns:
//   If any available  - 1
//   If none available - 0
//
LV_TTS_RETURN_CODE LV_TTS_IsServerAvailable();


// -----------------------------------------------------------------------------------------------
// LV_TTS_GetLastSSMLError
//     Provides the last SSML syntax error information, if any.
//
// Inputs:
//   client_handle           : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   error_buffer            : Pointer to a char buffer into which the SSML syntax error description will be copied
//   error_buffer_length     : Maximum length (in bytes) of error_buffer
//
// Returns:
//   LV_SUCCESS - if an error exists and the description was successfully copied into error_buffer
//   LV_FAILURE - if no errors exist or results are not yet available
//   LV_INVALID_TTS_HANDLE (if the client handle is invalid)
//   LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetLastSSMLError(HTTSCLIENT client_handle, char *error_buffer, int error_buffer_length);



// ===============================================================================================
//  Functions for Get/Set properties below
// ===============================================================================================


// -----------------------------------------------------------------------------------------------
// LV_TTS_SetPropertyEx
//     Set configuration properties for the LV_TTS interface
//
// Inputs:
//   client_handle           : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   int property            : one of several enumerated properties of int type, described below
//   int property_value_type : type of following parameter. legal values are:
//                             PROP_EX_VALUE_TYPE_INT, PROP_EX_VALUE_TYPE_INT_PTR,
//                             PROP_EX_VALUE_TYPE_STRING, PROP_EX_VALUE_TYPE_FLOAT_PTR
//   int *pvalue             : the value or a pointer to the value to be set for the property
//   int target              : the target scope of the property. legal values are:
//                             PROP_EX_TARGET_PORT (local), PROP_EX_TARGET_CLIENT (global)
//
// Properties
//   PROP_EX_SYNTHESIS_LANGUAGE     : a string value describing the language for synthesis (eg: "en-US")
//   PROP_EX_SYNTHESIS_SAMPLING_RATE: an integer that specifies the sampling rate that the audio is to be synthesized in
//   PROP_EX_SYNTH_SOUND_FORMAT     : an integer that indicates the format of the sound that the synthesized audio will be produced in
//   PROP_EX_SYNTH_PROSODY_PITCH    : a string value that sets the pitch of the audio produced
//   PROP_EX_SYNTH_PROSODY_CONTOUR  : a string value that sets the contour of the sound pitch produced
//   PROP_EX_SYNTH_PROSODY_RANGE    : a string value that sets the range of the sound produced
//   PROP_EX_SYNTH_PROSODY_RATE     : a string value that sets the speaking rate of the sound produced
//   PROP_EX_SYNTH_PROSODY_DURATION : a string value that sets the duration of the time it takes for the texts to play.
//   PROP_EX_SYNTH_PROSODY_VOLUME   : a string value that sets the volume of the audio produced
//   PROP_EX_SYNTH_VOICE_GENDER     : a string value that sets the gender of the voice timbre of the audio produced
//   PROP_EX_SYNTH_VOICE_AGE        : a string value that sets the age of the voice timbre of the audio produced
//   PROP_EX_SYNTH_VOICE_VARIANT    : a string value that sets the preferred voice variant of the audio produced
//   PROP_EX_SYNTH_VOICE_NAME       : a string value that sets the processor-specific voice name of the audio produced
//   PROP_EX_SYNTH_EMPHASIS_LEVEL   : a string value that sets the strength of emphasis of the audio produced
//   PROP_EX_LOG_TTS_EVENTS         : an integer (0 - off, 1 - on) that controls whether the application will save off call log files used with the LumenVox Speech Tuner
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_PROPERTY (if the specified property id is invalid)
//                LV_INVALID_TTS_HANDLE (if the client handle is invalid)
//                LV_INVALID_PROPERTY_VALUE_TYPE (value type is not valid for the specified property)
//                LV_INVALID_PROPERTY_TARGET (if the specified property is not valid for the specified target)
//                LV_FAILURE or LV_INVALID_API_PARAMETER (The provided pvalue is not valid, not permitted or not in acceptable range)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_SetPropertyEx(HTTSCLIENT client_handle, int property, int property_value_type, void *pvalue, int target);


// -----------------------------------------------------------------------------------------------
// LV_SRE_GetStringPropertyEx
//     Query configuration settings for string type options for the LV_TTS interface
//
// Inputs:
//   client_handle         : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   int property          : one of several enumerated properties of string type, described below
//   int target            : the target scope of the property. legal values are:
//                           PROP_EX_TARGET_PORT (local), PROP_EX_TARGET_CLIENT (global)
//   char *value_str       : buffer to store the queried value
//   int value_len_bytes:    size of the provided value_str buffer in bytes
//   int *actual_value_len : the actual amount of buffer in value_str used to store the queried value.
//                           If value_str is set to NULL or value_len_bytes set to zero, no copying is performed and
//                           the value returned in actual_value_len indicates the total size of value_str required to
//                           completely store the queried value, only if LV_SUCCESS is returned.
//
// Properties
//   PROP_EX_SYNTHESIS_LANGUAGE     : a string value describing the language for synthesis (eg: "en-US")
//   PROP_EX_SYNTH_PROSODY_PITCH    : a string value that sets the pitch of the audio produced
//   PROP_EX_SYNTH_PROSODY_CONTOUR  : a string value that sets the contour of the sound pitch produced
//   PROP_EX_SYNTH_PROSODY_RANGE    : a string value that sets the range of the sound produced
//   PROP_EX_SYNTH_PROSODY_RATE     : a string value that sets the speaking rate of the sound produced
//   PROP_EX_SYNTH_PROSODY_DURATION : a string value that sets the duration of the time it takes for the texts to play.
//   PROP_EX_SYNTH_PROSODY_VOLUME   : a string value that sets the volume of the audio produced
//   PROP_EX_SYNTH_VOICE_GENDER     : a string value that sets the gender of the voice timbre of the audio produced
//   PROP_EX_SYNTH_VOICE_AGE        : a string value that sets the age of the voice timbre of the audio produced
//   PROP_EX_SYNTH_VOICE_VARIANT    : a string value that sets the preferred voice variant of the audio produced
//   PROP_EX_SYNTH_VOICE_NAME       : a string value that sets the processor-specific voice name of the audio produced
//   PROP_EX_SYNTH_EMPHASIS_LEVEL   : a string value that sets the strength of emphasis of the audio produced
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_PROPERTY (if the specified property id is invalid)
//                LV_INVALID_TTS_HANDLE (if the client handle is invalid)
//                LV_INVALID_PROPERTY_TARGET (if the specified property is not valid for the specified target)
//                LV_FAILURE (if target is not valid or not supported)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetStringPropertyEx(HTTSCLIENT client_handle, int property, int target,
													   char *value_str, int value_len_bytes, int *actual_value_len);


// -----------------------------------------------------------------------------------------------
// LV_TTS_GetFloatPropertyEx
//     Query configuration settings for float type options for the LV_TTS interface
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   int property     : one of several enumerated properties of float type, described below
//   int target       : the target scope of the property. legal values are:
//                      PROP_EX_TARGET_PORT (local), 
//                      PROP_EX_TARGET_CLIENT (global)
//   float *pvalue    : address to store the queried value at
//
// Properties
//   None supported currently
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_PROPERTY (if the specified property id is invalid)
//                LV_INVALID_TTS_HANDLE (if the client handle is invalid)
//                LV_INVALID_PROPERTY_TARGET (if the specified property is not valid for the specified target)
//                LV_FAILURE (if target is not valid or not supported)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetFloatPropertyEx(HTTSCLIENT client_handle, int property, int target, float *pvalue);


// -----------------------------------------------------------------------------------------------
// LV_TTS_GetIntPropertyEx
//     Query configuration settings for int type options for the LV_TTS interface
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   int property     : one of several enumerated properties of int type, described below
//   int target       : the target scope of the property. legal values are:
//                      PROP_EX_TARGET_PORT (local),
//                      PROP_EX_TARGET_CLIENT (global)
//   int *pvalue      : address to store the queried value at
//
// Properties
//   PROP_EX_SYNTHESIS_SAMPLING_RATE: an integer that specifies the sampling rate that the audio is to be synthesized in
//   PROP_EX_SYNTH_SOUND_FORMAT     : format of the sound that the synthesized audio will be produced in
//   PROP_EX_LOG_TTS_EVENTS         : an integer (0 - off, 1 - on) that specifies whether the application will save off call log files
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_PROPERTY (if the specified property id is invalid)
//                LV_INVALID_TTS_HANDLE (if the client handle is invalid)
//                LV_INVALID_PROPERTY_TARGET (if the specified property is not valid for the specified target)
//                LV_FAILURE (if target is not valid or not supported)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetIntPropertyEx(HTTSCLIENT client_handle, int property, int target, int *pvalue);


// -----------------------------------------------------------------------------------------------
// LV_TTS_SetCustomCallGuid
//     This function is used to set the ID used in call logging. If not used, a globally unique id
//     will be generated automatically
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   call_guid        : Pointer to a valid string containing the custom guid to be used in call logging
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_TTS_HANDLE (if the client handle is invalid)
//                LV_INVALID_API_PARAMETER (if the input call_guid is not valid)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_SetCustomCallGuid(HTTSCLIENT client_handle, const char* call_guid);


// -----------------------------------------------------------------------------------------------
// LV_TTS_GetCallGuid
//     Query the ID used in call logging. It can be used in conjuction with
//     LV_TTS_SetCustomCallGuid() to link events across SRE ports and TTS ports to the same call
//     log. If no call guid exists at the time of invocation of this function, it simply returns
//     an empty string. 
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   call_guid        : Pointer to an empty buffer to be populated with the current guid used in call logging
//   call_guid_length : Length of above buffer
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_TTS_HANDLE (if the client handle is invalid)
//                LV_INVALID_API_PARAMETER (the input call_guid is not valid)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_GetCallGuid(HTTSCLIENT client_handle, char* call_guid, int call_guid_length);


// -----------------------------------------------------------------------------------------------
// LV_TTS_AddEvent
//     Add logging events to the call log file.
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   event_name       : Any non-NULL, non-zero length string
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_TTS_HANDLE (if the client handle is invalid) 
//                LV_INVALID_API_PARAMETER (if invalid parameter is detected)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_AddEvent(HTTSCLIENT client_handle, const char* event_name);


// -----------------------------------------------------------------------------------------------
// LV_TTS_AddFieldToRequest
//     Add a custom string to a named field in the call log file, that can be used to
//     filter/identify certain calls. This value persists until changed or the call ends.
//
// Inputs:
//   client_handle    : Handle to the TTS client object obtained from a previous call to LV_TTS_CreateClient
//   key_name         : Name of the Field which is to be added to the .callsre file
//   string_value     : String value to be assigned to the new field in the Callsre file
//
// Returns:
//   On success - LV_SUCCESS
//   On failure - LV_INVALID_TTS_HANDLE (if the client handle is invalid)
//                LV_INVALID_API_PARAMETER (if invalid parameter is detected)
//                LV_EXCEPTION (An exception occured while processing the request)
//
LV_TTS_RETURN_CODE LV_TTS_AddFieldToRequest(HTTSCLIENT client_handle, const char * key_name, const char * string_value);


#if defined(__cplusplus)
}	// extern "C"
#endif



#endif	/* __LV_TTS_H_2F731CF5_F3C6_4607_9AB6_9B1881D899DC__INCLUDED__ */

